@@ -1,12 +1,12 @@
-/* 
+/*
  * Gnome Chemistry Utils GOffice component
  * gogcpapp.cc
  *
  * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -51,37 +51,29 @@ void GOGcpApplication::ToggleMenu (G_GNUC_UNUSED const string& menuname, G_GNUC_
 {
 }
 
-gcu::Document *GOGcpApplication::ImportDocument (const string& mime_type, const char* data, int length)
+void GOGcpApplication::ImportDocument (GOGChemUtilsComponent *gogcu)
 {
 	gcp::Document *pDoc = NULL;
-	char *old_num_locale, *old_time_locale;
-	if (mime_type == "application/x-gchempaint") {
+	GOComponent *component = GO_COMPONENT (gogcu);
+	if (!strcmp (component->mime_type, "application/x-gchempaint")) {
 		xmlDocPtr xml;
-		if (!(xml = xmlParseMemory (data, length)) ||
+		if (!(xml = xmlParseMemory (component->data, component->length)) ||
 			xml->children == NULL ||
 			strcmp(reinterpret_cast <char const *> (xml->children->name), "chemistry")) {
 			xmlFreeDoc (xml);
-			return NULL;
+			return;
 		}
-		old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		setlocale (LC_NUMERIC, "C");
-		old_time_locale = g_strdup (setlocale (LC_TIME, NULL));
-		setlocale (LC_TIME, "C");
 		pDoc = new gcp::Document (this, false);
 		pDoc->GetView ()->CreateNewWidget ();
 		bool result = pDoc->Load (xml->children);
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
-		setlocale (LC_TIME, old_time_locale);
-		g_free (old_time_locale);
 		xmlFreeDoc (xml);
 		if (!result) {
 			delete pDoc;
-			return NULL;
+			return;
 		}
 	} else {
-	}	
-	return (pDoc);
+	}
+	gogcu->document = pDoc;
 }
 
 GtkWindow * GOGcpApplication::EditDocument (GOGChemUtilsComponent *gogcu)
@@ -114,7 +106,7 @@ bool GOGcpApplication::GetData (GOGChemUtilsComponent *gogcu, gpointer *data, in
 		* clearfunc = NULL;
 		return true;
 	}
-	
+
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale(LC_NUMERIC, "C");
 	old_time_locale = g_strdup (setlocale (LC_TIME, NULL));
@@ -184,7 +176,8 @@ void GOGcpApplication::OnFileNew (char const *Theme)
 	if (m_pActiveDoc && !m_pActiveDoc->GetView ()->PrepareUnselect ())
 		return;
 	g_snprintf (tmp, sizeof (tmp), _("Untitled %d"), m_NumWindow++);
-	new gcp::Window (this, Theme);
+	gcp::Window *win = new gcp::Window (this, Theme);
+	win->GetDocument ()->SetUseAtomColors (GetUseAtomColors ());
 }
 
 void GOGcpApplication::OnFileClose ()
