@@ -1,12 +1,12 @@
-/* 
+/*
  * Gnome Chemistry Utils GOffice component
  * gogcpapp.h
  *
- * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -37,7 +37,7 @@ public:
 	virtual GtkWindow* GetWindow();
 	virtual void ToggleMenu (const std::string& menuname, bool active);
 
-	gcu::Document *ImportDocument (const std::string& mime_type, const char* data, int length);
+	void ImportDocument (GOGChemUtilsComponent *gogcu);
 	GtkWindow *EditDocument (GOGChemUtilsComponent *gogcu);
 	bool GetData (GOGChemUtilsComponent *gogcu, gpointer *data, int *length, void (**clearfunc) (gpointer), gpointer *user_data);
 	void Render (GOGChemUtilsComponent *gogcu, cairo_t *cr, double width, double height);
@@ -47,6 +47,7 @@ public:
 	void OnFileClose ();
 
 	void NoMoreDocsEvent () {;}
+	gcu::ContentType GetContentType () {return gcu::ContentType2D;}
 
 private:
 	std::map<gcp::Document *, gpointer> m_Windows;
