@@ -2,11 +2,11 @@
  * Gnome Chemistry Utils GOffice component
  * gogcrystal.cc
  *
- * Copyright (C) 2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2010-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -24,16 +24,24 @@
 #include "gchemutils-priv.h"
 #include "gogcrystalapp.h"
 #include "gogcrystalwin.h"
+#include <gcr/atom.h>
 #include <gcr/document.h>
 #include <gcr/view.h>
+#include <gsf/gsf-input-memory.h>
 #include <glib/gi18n-lib.h>
 #include <cstring>
 
 using namespace gcu;
 using namespace std;
 
-GOGCrystalApplication::GOGCrystalApplication (): gcu::Application ("gcrystal"), GOGcuApplication ()
+static gcu::Object *CreateAtom ()
 {
+	return new gcr::Atom ();
+}
+
+GOGCrystalApplication::GOGCrystalApplication (): gcr::Application (), GOGcuApplication ()
+{
+		AddType ("atom", CreateAtom, gcu::AtomType);
 }
 
 GOGCrystalApplication::~GOGCrystalApplication ()
@@ -49,24 +57,32 @@ void GOGCrystalApplication::ToggleMenu (G_GNUC_UNUSED const string& menuname, G_
 {
 }
 
-gcu::Document *GOGCrystalApplication::ImportDocument (const string& mime_type, const char* data, int length)
+void GOGCrystalApplication::ImportDocument (GOGChemUtilsComponent *gogcu)
 {
 	gcr::Document *doc = NULL;
-	if (mime_type == "application/x-gcrystal") {
+	GOComponent *component = GO_COMPONENT (gogcu);
+	if (!strcmp (component->mime_type, "application/x-gcrystal")) {
 		xmlDocPtr xml;
-		if (!(xml = xmlParseMemory(data, length)) ||
+		if (!(xml = xmlParseMemory (component->data, component->length)) ||
 			xml->children == NULL ||
 			strcmp (reinterpret_cast <char const *> (xml->children->name), "crystal")) {
 			xmlFreeDoc (xml);
-			return NULL;
+			return;
 		}
 		doc = new gcr::Document (this);
 		gtk_widget_show_all (doc->GetView ()->GetWidget ());
 		doc->ParseXMLTree (xml->children);
 		xmlFreeDoc (xml);
-	} else {
+	} else if (!strcmp (component->mime_type, "chemical/x-cif")) {
+		doc = new gcr::Document (this);
+		GsfInput *input = gsf_input_memory_new (reinterpret_cast < guint8 const * > (component->data), component->length, false);
+		Load (input, component->mime_type, doc);
+		doc->Loaded ();
+		doc->Update ();
+		g_object_unref (input);
 	}
-	return (doc);
+	gogcu->document = doc;
+	component->resizable = true;
 }
 
 GtkWindow * GOGCrystalApplication::EditDocument (GOGChemUtilsComponent *gogcu)
@@ -88,23 +104,18 @@ GtkWindow * GOGCrystalApplication::EditDocument (GOGChemUtilsComponent *gogcu)
 
 bool GOGCrystalApplication::GetData (GOGChemUtilsComponent *gogcu, gpointer *data, int *length, void (**clearfunc) (gpointer), G_GNUC_UNUSED gpointer *user_data)
 {
-//	gcp::Document *doc = static_cast <gcp::Document *> (gogcu->document);
+	gcr::Document *doc = static_cast <gcr::Document *> (gogcu->document);
+	GOComponent *component = GO_COMPONENT (gogcu);
 	bool result = false;
-/*	xmlDocPtr xml = NULL;
-	char *old_num_locale, *old_time_locale;
+	xmlDocPtr xml = NULL;
 
-	if (!doc || !doc->HasChildren ()) {
+	if (!doc || doc->GetEmpty ()) {
 		*data = NULL;
 		*length = 0;
-		* clearfunc = NULL;
+		*clearfunc = NULL;
 		return true;
 	}
 
-	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale(LC_NUMERIC, "C");
-	old_time_locale = g_strdup (setlocale (LC_TIME, NULL));
-	setlocale (LC_TIME, "C");
-
 	try {
 		xml = doc->BuildXMLTree ();
 		xmlChar *mem;
@@ -115,17 +126,20 @@ bool GOGCrystalApplication::GetData (GOGChemUtilsComponent *gogcu, gpointer *dat
 		*length = size;
 		*clearfunc = xmlFree;
 		result = true;
+		if (strcmp (component->mime_type, "application/x-gcrystal")) {
+			g_free (component->mime_type);
+			component->mime_type = g_strdup ("application/x-gcrystal");
+		}
 	}
 	catch (int) {
 		if (xml)
 			xmlFreeDoc (xml);
 		xml = NULL;
+		*data = NULL;
+		*length = 0;
+		*clearfunc = NULL;
 		result = false;
 	}
-	setlocale (LC_NUMERIC, old_num_locale);
-	g_free (old_num_locale);
-	setlocale (LC_TIME, old_time_locale);
-	g_free (old_time_locale);*/
 
 	return result;
 }
@@ -136,36 +150,13 @@ void GOGCrystalApplication::Render (GOGChemUtilsComponent *gogcu, cairo_t *cr, d
 	doc->GetView ()->RenderToCairo (cr, width, height, false);
 }
 
-void GOGCrystalApplication::UpdateBounds (GOGChemUtilsComponent *gogcu)
+void GOGCrystalApplication::UpdateBounds (GOGChemUtilsComponent *)
 {
-/*	gcp::Document *doc = static_cast <gcp::Document *> (gogcu->document);
-	gcp::Theme *pTheme = doc->GetTheme ();
-	GtkWidget *w = doc->GetWidget ();
-	gccv::Rect rect;
-	gcp::WidgetData *pData = (gcp::WidgetData*) g_object_get_data (G_OBJECT (w), "data");
-	pData->GetObjectBounds (doc, &rect);
-	double y = doc->GetYAlign ();
-	y += doc->GetView ()->GetBaseLineOffset ();
-	y *= pTheme->GetZoomFactor ();
-	if (rect.x0 || rect.y0)
-		doc->Move (- rect.x0 / pTheme->GetZoomFactor (), - rect.y0 / pTheme->GetZoomFactor ());
-	doc->GetView ()->Update (doc);
-	if (y < rect.y0)
-		y = rect.y1;
-	// assuming 96 dpi, setting dimensions as inches.
-	gogcu->parent.ascent = (y - rect.y0) / 96;
-	gogcu->parent.descent = (rect.y1 - y) / 96;
-	gogcu->parent.width = (rect.x1 - rect.x0) / 96;
-	gogcu->parent.height = gogcu->parent.ascent + gogcu->parent.descent;*/
 }
 
-void GOGCrystalApplication::OnFileNew (char const *Theme)
+gcr::Document *GOGCrystalApplication::OnFileNew ()
 {
-/*	gchar tmp[32];
-	if (m_pActiveDoc && !m_pActiveDoc->GetView ()->PrepareUnselect ())
-		return;
-	g_snprintf (tmp, sizeof (tmp), _("Untitled %d"), m_NumWindow++);
-	new gcp::Window (this, Theme);*/
+	return m_pActiveDoc = new gcr::Document (this);
 }
 
 void GOGCrystalApplication::OnFileClose ()
