@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Chemistry Utils
- * gccv/line.cc 
+ * gccv/line.cc
  *
  * Copyright (C) 2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -84,11 +84,11 @@ double Line::Distance (double x, double y, Item **item) const
 void Line::Draw (cairo_t *cr, G_GNUC_UNUSED bool is_vector) const
 {
 	cairo_operator_t op = GetOperator ();
-	cairo_set_line_width (cr, GetLineWidth ());
+	ApplyLine (cr);
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
 	cairo_move_to (cr, m_xstart, m_ystart);
 	cairo_line_to (cr, m_xend, m_yend);
-	GOColor color = GetLineColor ();
+	GOColor color = GetEffectiveLineColor ();
 	if (op == CAIRO_OPERATOR_CLEAR || op == CAIRO_OPERATOR_SOURCE) {
 		cairo_content_t content = cairo_surface_get_content (cairo_get_target (cr));
 		if (!(content & CAIRO_CONTENT_ALPHA))
@@ -96,6 +96,7 @@ void Line::Draw (cairo_t *cr, G_GNUC_UNUSED bool is_vector) const
 	}
 	cairo_set_source_rgba (cr, GO_COLOR_TO_CAIRO (color));
 	cairo_stroke (cr);
+	cairo_restore (cr);
 }
 
 void Line::UpdateBounds ()
