@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Chemistry Utils
  * gccv/text-tag.cc
  *
- * Copyright (C) 2008-2009 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2008-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,14 +21,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
- 
+
 #include "config.h"
 #include "text-tag.h"
 #include <list>
 #include <map>
 
 namespace gccv {
-	
+
 ////////////////////////////////////////////////////////////////////////////////
 // Base tag class
 
@@ -68,6 +68,36 @@ bool TextTag::Order (TextTag *first, TextTag *last)
 	return first->GetTag () < last->GetTag ();
 }
 
+TextTag *TextTag::Restrict (TextTag *tag)
+{
+	if (tag->GetTag () == GetTag () && tag->GetEndIndex () > GetStartIndex () && tag->GetStartIndex () < GetEndIndex ()) {
+		if (*tag == *this) {
+			if (m_StartIndex > tag->GetStartIndex ())
+				m_StartIndex = tag->GetStartIndex ();
+			if (m_EndIndex < tag->GetEndIndex ())
+				m_EndIndex = tag->GetEndIndex ();
+			tag->SetEndIndex (m_StartIndex); // makes tag invalid
+			return NULL;
+		}
+		if (tag->GetEndIndex () > GetEndIndex ()) {
+			if (tag->GetStartIndex () < GetStartIndex ()) {
+				// split tag
+				TextTag *new_tag = tag->Duplicate ();
+				new_tag->SetStartIndex (m_EndIndex);
+				new_tag->SetEndIndex (tag->GetEndIndex ());
+				tag->SetEndIndex (m_StartIndex);
+				return new_tag;
+			}
+			tag->SetStartIndex (m_EndIndex);
+			return NULL;
+		} else {
+			tag->SetEndIndex (m_StartIndex);
+			return NULL;
+		}
+	}
+	return NULL;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Font family tag class
 
@@ -275,9 +305,10 @@ TextTag *StretchTextTag::Duplicate () const
 ////////////////////////////////////////////////////////////////////////////////
 // Font underline tag class
 
-UnderlineTextTag::UnderlineTextTag (PangoUnderline underline):
+UnderlineTextTag::UnderlineTextTag (TextDecoration underline, GOColor color):
 	TextTag (Underline),
-	m_Underline (underline)
+	m_Underline (underline),
+	m_Color (color)
 {
 }
 
@@ -301,12 +332,43 @@ TextTag *UnderlineTextTag::Duplicate () const
 	return new UnderlineTextTag (m_Underline);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Font overline tag class
+
+OverlineTextTag::OverlineTextTag (TextDecoration overline, GOColor color):
+	TextTag (Overline),
+	m_Overline (overline),
+	m_Color (color)
+{
+}
+
+OverlineTextTag::~OverlineTextTag ()
+{
+}
+
+void OverlineTextTag::Filter (G_GNUC_UNUSED PangoAttrList *l, G_GNUC_UNUSED unsigned start, G_GNUC_UNUSED unsigned end)
+{
+}
+
+bool OverlineTextTag::operator== (TextTag const& tag) const
+{
+	if (tag.GetTag () != Overline)
+		return false;
+	return static_cast <OverlineTextTag const&> (tag).m_Overline == m_Overline;
+}
+
+TextTag *OverlineTextTag::Duplicate () const
+{
+	return new OverlineTextTag (m_Overline);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Font strikethrough tag class
 
-StrikethroughTextTag::StrikethroughTextTag (bool strikethrough):
+StrikethroughTextTag::StrikethroughTextTag (TextDecoration strikethrough, GOColor color):
 	TextTag (Strikethrough),
-	m_Strikethrough (strikethrough)
+	m_Strikethrough (strikethrough),
+	m_Color (color)
 {
 }
 
@@ -345,7 +407,8 @@ ForegroundTextTag::~ForegroundTextTag ()
 
 void ForegroundTextTag::Filter (PangoAttrList *l, unsigned start, unsigned end)
 {
-	PangoAttribute *attr = pango_attr_foreground_new (UINT_RGBA_R (m_Color) * 0x101, UINT_RGBA_G (m_Color) * 0x101, UINT_RGBA_B (m_Color) * 0x101);
+	// this might be not enough since we use a global attribute with SetColor().
+	PangoAttribute *attr = pango_attr_foreground_new (GO_COLOR_UINT_R (m_Color) * 0x101, GO_COLOR_UINT_G (m_Color) * 0x101, GO_COLOR_UINT_B (m_Color) * 0x101);
 	attr->start_index = start;
 	attr->end_index = end;
 	pango_attr_list_insert (l, attr);
@@ -378,7 +441,7 @@ BackgroundTextTag::~BackgroundTextTag ()
 
 void BackgroundTextTag::Filter (PangoAttrList *l, unsigned start, unsigned end)
 {
-	PangoAttribute *attr = pango_attr_background_new (UINT_RGBA_R (m_Color) * 0x101, UINT_RGBA_G (m_Color) * 0x101, UINT_RGBA_B (m_Color) * 0x101);
+	PangoAttribute *attr = pango_attr_background_new (GO_COLOR_UINT_R (m_Color) * 0x101, GO_COLOR_UINT_G (m_Color) * 0x101, GO_COLOR_UINT_B (m_Color) * 0x101);
 	attr->start_index = start;
 	attr->end_index = end;
 	pango_attr_list_insert (l, attr);
@@ -547,9 +610,8 @@ TextTag *PositionTextTag::Duplicate () const
 ////////////////////////////////////////////////////////////////////////////////
 // Position tag class (normal, subscript, superscript,...)
 
-NewLineTextTag::NewLineTextTag (double interline):
-	TextTag (NewLine),
-	m_Interline (interline)
+NewLineTextTag::NewLineTextTag ():
+	TextTag (NewLine)
 {
 	m_NewLine = true;
 }
@@ -557,22 +619,20 @@ NewLineTextTag::NewLineTextTag (double interline):
 NewLineTextTag::~NewLineTextTag ()
 {
 }
-	
+
 void NewLineTextTag::Filter (G_GNUC_UNUSED PangoAttrList *l, G_GNUC_UNUSED unsigned start, G_GNUC_UNUSED unsigned end)
 {
 	// Nothing to do
 }
 
-bool NewLineTextTag::operator== (TextTag const& tag) const
+bool NewLineTextTag::operator== (G_GNUC_UNUSED TextTag const& tag) const
 {
-	if (tag.GetTag () != NewLine)
-		return false;
-	return static_cast <NewLineTextTag const&> (tag).m_Interline == m_Interline;
+	return false;
 }
 
 TextTag *NewLineTextTag::Duplicate () const
 {
-	return new NewLineTextTag (m_Interline);
+	return new NewLineTextTag ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
