@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -26,6 +26,7 @@
 #include "arrow.h"
 #include "document.h"
 #include "settings.h"
+#include "step.h"
 #include "view.h"
 #include "widgetdata.h"
 #include <gcu/objprops.h>
@@ -44,10 +45,17 @@ Arrow::Arrow (TypeId Type):
 	Object (Type),
 	gccv::ItemClient ()
 {
+	m_Start = m_End = NULL;
 }
 
 Arrow::~Arrow ()
 {
+	if (IsLocked ())
+		return;
+	if (m_Start)
+		m_Start->RemoveArrow (this, m_End);
+	if (m_End)
+		m_End->RemoveArrow (this, m_Start);
 }
 
 bool Arrow::Save (xmlDocPtr xml, xmlNodePtr node) const
@@ -142,6 +150,7 @@ void Arrow::SetCoords (double xstart, double ystart, double xend, double yend)
 	m_y = ystart;
 	m_width = xend - xstart;
 	m_height = yend - ystart;
+	m_Length = sqrt (m_width * m_width + m_height *m_height);
 }
 
 bool Arrow::GetCoords (double* xstart, double* ystart, double* xend, double* yend) const
@@ -212,4 +221,31 @@ std::string Arrow::Name ()
 	return _("Arrow");
 }
 
+void Arrow::Reverse ()
+{
+	Step *step = m_Start;
+	m_Start = m_End;
+	m_End = step;
+	m_x = m_x + m_width;
+	m_y = m_y + m_height;
+	m_width = - m_width;
+	m_height = - m_height;
+}
+
+void Arrow::RemoveStep (Step *step)
+{
+	if (step == m_Start)
+		m_Start = NULL;
+	else if (step == m_End)
+		m_End = NULL;
+}
+
+void Arrow::OnLoaded ()
+{
+	if (m_Start)
+		m_Start->AddArrow (this, m_End);
+	if (m_End)
+		m_End->AddArrow (this, m_Start);
+}
+
 }	//	namespace gcp
