@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,25 +25,33 @@
 #include "config.h"
 #include "brackets.h"
 #include <gccv/canvas.h>
+#include <gcp/application.h>
 #include <gcp/atom.h>
 #include <gcp/bond.h>
 #include <gcp/document.h>
 #include <gcp/fragment.h>
 #include <gcp/mechanism-step.h>
+#include <gcp/mesomery.h>
 #include <gcp/reaction-step.h>
 #include <gcp/settings.h>
+#include <gcp/text.h>
 #include <gcp/theme.h>
+#include <gcp/tool.h>
 #include <gcp/view.h>
 #include <gcp/widgetdata.h>
 #include <gcu/application.h>
 #include <gcu/document.h>
+#include <gcugtk/ui-manager.h>
 #include <gccv/canvas.h>
 #include <gccv/structs.h>
 #include <sstream>
 #include <cstring>
+#include <glib/gi18n-lib.h>
 
 namespace gcp {
 
+extern xmlDocPtr pXmlDoc;
+
 gcu::TypeId BracketsType = gcu::NoType;
 static gcu::Object *last_loaded;
 
@@ -54,6 +62,7 @@ Brackets::Brackets (gccv::BracketsTypes type): gcu::Object (BracketsType), ItemC
 	m_Valid = false;
 	m_Used = gccv::BracketsBoth;
 	m_Content = BracketContentInvalid;
+	m_Decorations = BracketDecorationNone;
 }
 
 Brackets::~Brackets ()
@@ -74,6 +83,7 @@ void Brackets::OnLoaded ()
 		m_EmbeddedObjects.insert (last_loaded);
 		last_loaded->Link (this);
 		last_loaded = NULL;
+		UpdateItem ();
 	}
 }
 
@@ -83,6 +93,9 @@ void Brackets::AddItem ()
 		return;
 	Document *doc = static_cast <Document*> (GetDocument ());
 	View *view = doc->GetView ();
+	Theme *theme= doc->GetTheme ();
+	gcu::Object *child;
+	std::map < std::string, gcu::Object * >::iterator i;
 	if (!m_FontDesc.length ()) {
 		char *desc = pango_font_description_to_string (view->GetPangoFontDesc ());
 		m_FontDesc = desc;
@@ -90,9 +103,8 @@ void Brackets::AddItem ()
 	}
 	gccv::Rect rect;
 	if (m_EmbeddedObjects.size () == 1 && GetParent () == *m_EmbeddedObjects.begin ()) {
-		std::map < std::string, gcu::Object * >::iterator i;
 		rect.x0 = go_nan;
-		gcu::Object *parent = GetParent (), *child;
+		gcu::Object *parent = GetParent ();
 		for (child = parent->GetFirstChild (i); child; child = parent->GetNextChild (i)) {
 			if (child == this)
 				continue;
@@ -106,6 +118,17 @@ void Brackets::AddItem ()
 	gccv::Brackets *item = new gccv::Brackets (view->GetCanvas ()->GetRoot (), m_Type, m_Used, m_FontDesc.c_str (), rect.x0, rect.y0, rect.x1, rect.y1, this);
 	item->SetLineColor ((view->GetData ()->IsSelected (this))? SelectColor: GO_COLOR_BLACK);
 	m_Item = item;
+	// force children update if any
+	item->GetBounds (rect.x0, rect.y0, rect.x1, rect.y1);
+	for (child = GetFirstChild (i); child; child = GetNextChild (i)) {
+		Text *text = dynamic_cast < Text * > (child);
+		if (text && text->GetGlobalTag () == StoichiometryTag) {
+			text->SetCoords (rect.x1 / theme->GetZoomFactor (),
+			                 (rect.y1 + theme->GetFontSize () / 3. / PANGO_SCALE) / theme->GetZoomFactor ());
+			if (text->GetItem ())
+				text->UpdateItem ();
+		}
+	}
 }
 
 bool Brackets::Load (xmlNodePtr node)
@@ -132,15 +155,14 @@ bool Brackets::Load (xmlNodePtr node)
 		g_strfreev (ids);
 		xmlFree (buf);
 	}
-	return true;
+	return gcu::Object::Load (node);
 }
 
 xmlNodePtr Brackets::Save (xmlDocPtr xml) const
 {
 	if (m_EmbeddedObjects.size () == 0)
 		return NULL;
-	xmlNodePtr node = xmlNewDocNode (xml, NULL, (xmlChar*) "brackets", NULL);
-	SaveId (node);
+	xmlNodePtr node = gcu::Object::Save (xml);//xmlNewDocNode (xml, NULL, (xmlChar*) "brackets", NULL);
 	char const *type = NULL;
 	switch (m_Type) {
 	case gccv::BracketsTypeNormal:
@@ -168,6 +190,8 @@ xmlNodePtr Brackets::Save (xmlDocPtr xml) const
 
 void Brackets::SetSelected (int state)
 {
+	if (!m_Item)
+		return;
 	GOColor color;
 	switch (state) {
 	case SelStateUnselected:
@@ -199,21 +223,20 @@ void Brackets::SetEmbeddedObjects (std::set < gcu::Object * > objects)
 									   end = objects.end ();
 	std::set <gcu::TypeId> const &rules = GetApplication ()->GetRules (BracketsType, gcu::RuleMayContain);
 
-	bool ok;
+	bool ok = true;
 	if ((ok = objects.size () == 1)) {
 		obj = *i;
 		gcu::TypeId type = obj->GetType ();
 		if (type == gcu::MoleculeType)
 			m_Content = BracketContentMolecule;
-		else if (type == gcp::ReactionStepType || type == gcp::MechanismStepType || rules.find (type) != rules.end ())
+		else if (type == gcp::ReactionStepType || type == gcp::MechanismStepType ||
+		         type == gcu::MesomeryType || rules.find (type) != rules.end ())
 			m_Content = BracketContentGroup;
 		else
-			ok =false;
+			ok = false;
 		if (m_Used == gccv::BracketsBoth)
 			m_Decorations = BracketSuperscript;
 	}
-	if (m_Used != gccv::BracketsBoth)
-		return;
 	if (!ok) {
 		obj = (*i)->GetMolecule ();
 		if (obj != NULL) {
@@ -226,8 +249,9 @@ void Brackets::SetEmbeddedObjects (std::set < gcu::Object * > objects)
 			m_Content = BracketContentFragment;
 		} else
 			return; // may be we are missing some cases where the enclosed group is valid
+		if (m_Used == gccv::BracketsBoth)
+			m_Decorations = BracketSubscript;
 	}
-	m_Decorations = BracketSubscript;
 
 	SetParent (obj);
 	// unset existing links
@@ -285,4 +309,91 @@ bool Brackets::ConnectedAtoms (std::set < gcu::Object * > const &objects)
 	return objects.size () == test.size () + nb;
 }
 
+static void on_stoichiometry_add (Brackets *brackets) {
+	Document *pDoc = dynamic_cast <Document*> (brackets->GetDocument ());
+	Application * pApp = pDoc->GetApplication ();
+	View *pView = pDoc->GetView ();
+	Theme *pTheme= pDoc->GetTheme ();
+	WidgetData *pData = (WidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
+	gccv::Rect rect;
+	Operation *op = pDoc->GetNewOperation (GCP_MODIFY_OPERATION);
+	op->AddNode (brackets->GetGroup ()->Save (pXmlDoc), 0);
+	pData->GetObjectBounds (brackets, &rect);
+	double x = rect.x1 / pTheme->GetZoomFactor (),
+		y = (rect.y1 + pTheme->GetFontSize () / 3. / PANGO_SCALE) / pTheme->GetZoomFactor ();
+	Text *text = new Text ((StoichiometryTag)? StoichiometryTag:(StoichiometryTag = gccv::TextTag::RegisterTagType ()), x, y);
+	text->SetAnchor (gccv::AnchorSouthWest);
+	brackets->AddChild (text);
+	pDoc->AddObject (text);
+	Tool *tool = pApp->GetTool ("Text");
+	brackets->GetParent ()->EmitSignal (OnChangedSignal);
+	pApp->ActivateTool ("Text", true);
+	tool->OnClicked (pView, text, rect.x1, rect.y1, 0);
+}
+
+static void on_superscript_add (Brackets *brackets) {
+	Document *pDoc = dynamic_cast <Document*> (brackets->GetDocument ());
+	Application * pApp = pDoc->GetApplication ();
+	View *pView = pDoc->GetView ();
+	Theme *pTheme= pDoc->GetTheme ();
+	WidgetData *pData = (WidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
+	gccv::Rect rect;
+	Operation *op = pDoc->GetNewOperation (GCP_MODIFY_OPERATION);
+	op->AddNode (brackets->GetGroup ()->Save (pXmlDoc), 0);
+	pData->GetObjectBounds (brackets, &rect);
+	double x = rect.x1 / pTheme->GetZoomFactor (),
+		y = rect.y0 / pTheme->GetZoomFactor ();
+	Text *text = new Text (x, y);
+	text->SetAnchor (gccv::AnchorNorthWest);
+	brackets->AddChild (text);
+	pDoc->AddObject (text);
+	Tool *tool = pApp->GetTool ("Text");
+	brackets->GetParent ()->EmitSignal (OnChangedSignal);
+	pApp->ActivateTool ("Text", true);
+	tool->OnClicked (pView, text, rect.x1, rect.y0, 0);
+}
+
+bool Brackets::BuildContextualMenu (gcu::UIManager *UIManager, Object *object, double x, double y)
+{
+	bool result = false;
+	if (!HasChildren () && m_Decorations != BracketDecorationNone && m_Used == gccv::BracketsBoth) {
+		result = true;
+		if (m_Decorations & BracketSubscript) {
+			GtkUIManager *uim = static_cast < gcugtk::UIManager * > (UIManager)->GetUIManager ();
+			GtkActionGroup *group = gtk_action_group_new ("bracket");
+			GtkAction *action;
+			action = gtk_action_new ("Brackets", _("Brackets"), NULL, NULL);
+			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
+			action = gtk_action_new ("Stoich", _("Add stoichiometry"), NULL, NULL);
+			g_signal_connect_swapped (action, "activate", G_CALLBACK (on_stoichiometry_add), this);
+			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
+			gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Brackets'><menuitem action='Stoich'/></menu></popup></ui>", -1, NULL);
+			gtk_ui_manager_insert_action_group (uim, group, 0);
+			g_object_unref (group);
+		} else if (m_Decorations & BracketSuperscript) {
+			GtkUIManager *uim = static_cast < gcugtk::UIManager * > (UIManager)->GetUIManager ();
+			GtkActionGroup *group = gtk_action_group_new ("bracket");
+			GtkAction *action;
+			action = gtk_action_new ("Brackets", _("Brackets"), NULL, NULL);
+			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
+			action = gtk_action_new ("Super", _("Add superscript"), NULL, NULL);
+			g_signal_connect_swapped (action, "activate", G_CALLBACK (on_superscript_add), this);
+//			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
+//			gtk_ui_manager_add_ui_from_string (uim, "<ui><popup><menu action='Brackets'><menuitem action='Super'/></menu></popup></ui>", -1, NULL);
+			gtk_ui_manager_insert_action_group (uim, group, 0);
+			g_object_unref (group);
+			result = false; // remove when we support superscripts
+		}
+	}
+	return result || Object::BuildContextualMenu (UIManager, object, x, y);
+}
+
+void Brackets::Transform2D (gcu::Matrix2D&, double, double)
+{
+}
+
 }
