@@ -743,11 +743,12 @@ void Molecule::OnLoaded ()
 	// First check if we have only one molecule or split, this might happen with
 	// cml (root node being molecule) or malformed files.
 	if (m_Atoms.size () > 1) {
+		/* FIXME: what should be done with fragments? */
 		std::set < gcu::Atom * > ConnectedAtoms;
 		std::list < gcu::Atom * >:: iterator i;
 		while (1) {
 			BuildConnectivity (GetFirstAtom (i), ConnectedAtoms);
-			if (m_Atoms.size () == ConnectedAtoms.size ())
+			if (m_Atoms.size () + m_Fragments.size () == ConnectedAtoms.size ())
 				break;
 			// now split the molecule
 			Molecule *new_mol = new Molecule (static_cast < Atom * > (GetFirstAtom (i)));
