@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
- * molecule.h 
+ * molecule.h
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2012 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,10 +27,14 @@
 
 #include "fragment.h"
 #include <list>
-#include <gcu/molecule.h>
-#include <openbabel/mol.h>
+#include <gcugtk/molecule.h>
 
 /*!\file*/
+
+namespace gcu {
+class UIManager;
+}
+
 namespace gcp {
 
 class Bond;
@@ -38,8 +42,9 @@ class Bond;
 /*!\class Molecule gcp/molecule.h
 \brief GChemPaint molecule class.
 */
-class Molecule: public gcu::Molecule
+class Molecule: public gcugtk::Molecule
 {
+friend class MoleculePrivate;
 public:
 /*!
 The default constructor. Builds a new empty molecule.
@@ -139,7 +144,7 @@ Used to retrieve the y coordinate for alignment.
 */
 	double GetYAlign ();
 /*!
-@param UIManager the GtkUIManager to populate.
+@param UIManager the gcu::UIManager to populate.
 @param object the Object on which occured the mouse click.
 @param x x coordinate of the mouse click.
 @param y y coordinate of the mouse click.
@@ -147,7 +152,7 @@ Used to retrieve the y coordinate for alignment.
 This method is called to build the contextual menu for the molecule.
 @return true if something is added to the UIManager, false otherwise.
 */
-	bool BuildContextualMenu (GtkUIManager *UIManager, gcu::Object *object, double x, double y);
+	bool BuildContextualMenu (gcu::UIManager *UIManager, gcu::Object *object, double x, double y);
 /*!
 @param Signal the appropriate SignalId
 @param Child the child which emitted the signal or NULL
@@ -158,11 +163,6 @@ Signals the molecule that at least one of its children changed.
 */
 	bool OnSignal (gcu::SignalId Signal, gcu::Object *Child);
 /*!
-Opens a Ghemical window with a copy of the molecule trying to preserve the
-stereochemistry as far as possible.
-*/
-	void ExportToGhemical ();
-/*!
 @param child the child used for alignment.
 
 Selects the child used for alignment, which might be an atom or group of atoms,
@@ -175,43 +175,6 @@ a call to their gcu::Object::GetYAlign() method will be used.
 */
 	std::string GetAlignmentId () {return (m_Alignment)? m_Alignment->GetId (): "";}
 /*!
-@param Mol the OpenBabel molecule used for export.
-
-Builds an OpenBabel molecule from this instance.
-The new molecule is built trying to guess appropriate z coordinates so that
-stereochemistry is preserved.
-*/
-	void BuildOBMol (OpenBabel::OBMol &Mol);
-/*!
-@param Mol the OpenBabel molecule used for export.
-
-Builds a 2D OpenBabel molecule from this instance.
-
-*/
-	void BuildOBMol2D (OpenBabel::OBMol &Mol);
-/*!
-Builds the InChI for the molecule if necessary and opens a new StringDlg instance
-displaying it.
-*/
-	void ShowInChI ();
-/*!
-Builds the InChI for the molecule.
-*/
-	void BuildInChI ();
-/*!
-Builds the SMILES representation for the molecule and opens a new StringDlg
-instance displaying it.
-*/
-	void BuildSMILES ();
-/*!
-@param uri_start the first part of the URI to open.
-@param uri_end the last part of the URI to open.
-
-Opens a web browser with an URI constructed from both arguments and the InChI
-for the molecule ntercalated between \a uri_start and \a uri_end.
-*/
-	void ShowWebBase (char const *uri_start, char const *uri_end);
-/*!
 Opens GChemCalc with the formula for the molecule. Molecules with fragments
 are not currently supported.
 */
@@ -224,11 +187,6 @@ that they are crossing.
 */
 	void CheckCrossings (Bond *pBond);
 /*!
-@return the InChI. The returned string should not be freed (it's a const char*).
-Molecules with fragments are not currently supported.
-*/
-	char const *GetInChI ();
-/*!
 @return the raw formula as a string. Molecules with fragments
 are not currently supported.
 */
@@ -243,11 +201,28 @@ only, whatever their real atomic composition.
 */
 	unsigned GetAtomsNumber () const;
 
+/*!
+@return the mean of the bond lengths for the molecule.
+*/
+	double GetMeanBondLength () const;
+
+/*!
+@param atom a chiral atom
+
+Adds \a atom to the list of chiral atoms.
+*/
+	void AddChiralAtom (Atom *atom) {m_ChiralAtoms.insert (atom);}
+
+/*!
+@param atom an atom
+@return true if \a atom is a chirality center.
+*/
+	bool AtomIsChiral (Atom *atom) const;
+
 private:
-	std::list<Fragment*> m_Fragments;
+	std::list< Fragment * > m_Fragments;
+	std::set < Atom * > m_ChiralAtoms;
 	gcu::Object *m_Alignment;
-	std::string m_InChI;
-	bool m_Changed;
 	bool m_IsResidue;
 };
 
