@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
- * text.cc 
+ * text.cc
  *
  * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -28,6 +28,7 @@
 #include "view.h"
 #include "document.h"
 #include "application.h"
+#include "fragment.h"
 #include "settings.h"
 #include "theme.h"
 #include "tool.h"
@@ -50,6 +51,7 @@ namespace gcp {
 Text::Text ():
 	TextObject (TextType),
 	m_Anchor (gccv::AnchorLineWest),
+	m_GlobalTag (gccv::Invalid),
 	m_Interline (0.),
 	m_Justification (GTK_JUSTIFY_LEFT)
 {
@@ -58,6 +60,16 @@ Text::Text ():
 Text::Text (double x, double y):
 	TextObject (x, y, TextType),
 	m_Anchor (gccv::AnchorLineWest),
+	m_GlobalTag (gccv::Invalid),
+	m_Interline (0.),
+	m_Justification (GTK_JUSTIFY_LEFT)
+{
+}
+
+Text::Text (gccv::Tag tag, double x, double y):
+	TextObject (x, y, TextType),
+	m_Anchor (gccv::AnchorLineWest),
+	m_GlobalTag (tag),
 	m_Interline (0.),
 	m_Justification (GTK_JUSTIFY_LEFT)
 {
@@ -348,7 +360,7 @@ bool SaveStruct::Save (xmlDocPtr xml, xmlNodePtr node, unsigned &index, string c
 		double rise = static_cast <gccv::RiseTextTag *> (m_tag)->GetRise ();
 		if (rise != 0) {
 			child = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ((rise > 0)? "sup": "sub"), NULL);
-			char *buf = g_strdup_printf ("%g", fabs (rise));
+			char *buf = g_strdup_printf ("%g", fabs (rise) / PANGO_SCALE);
 			xmlNewProp (child, reinterpret_cast <xmlChar const *> ("height"), reinterpret_cast <xmlChar const *> (buf));
 			g_free (buf);
 			xmlAddChild (node, child);
@@ -450,11 +462,18 @@ xmlNodePtr Text::Save (xmlDocPtr xml) const
 	default:
 		break;
 	}
+	if (m_GlobalTag != gccv::Invalid) {
+		if (m_GlobalTag == StoichiometryTag)
+			xmlNewProp (node, reinterpret_cast < xmlChar const * > ("role"), reinterpret_cast < xmlChar const * > ("stoichiometry"));
+		// TODO: support other tags, such as chemistry
+		xmlNodeAddContent (node, reinterpret_cast <xmlChar const *> (m_buf.c_str ()));
+		return node;
+	}
 	if (m_Interline > 0.) {
 		char *buf = g_strdup_printf ("%g", m_Interline);
 		xmlNewProp (node, reinterpret_cast <xmlChar const *> ("interline"), reinterpret_cast <xmlChar *> (buf));
 		g_free (buf);
-		
+
 	}
 	unsigned i = 0;
 	SaveStruct *head = NULL, *cur;
@@ -558,6 +577,16 @@ bool Text::Load (xmlNodePtr node)
 		m_Interline = strtod (reinterpret_cast <char *> (buf), NULL);
 		xmlFree (buf);
 	}
+	buf = xmlGetProp (node, (xmlChar const *) "role");
+	if (buf) {
+		if (!strcmp (reinterpret_cast <char *> (buf), "stoichiometry")) {
+			if (StoichiometryTag == gccv::Invalid)
+				StoichiometryTag = gccv::TextTag::RegisterTagType ();
+			m_GlobalTag = StoichiometryTag;
+			m_Anchor = gccv::AnchorSouthWest;
+		}
+		xmlFree (buf);
+	}
 	xmlNodePtr child;
 	m_bLoading = true;
 	child = node->children;
@@ -721,7 +750,7 @@ bool Text::LoadNode (xmlNodePtr node, unsigned &pos, int cur_size)
 	} else if (!strcmp ((const char*) node->name, "sub")) {
 		buf = (char*) xmlGetProp (node, (xmlChar*) "height");
 		if (buf) {
-			int rise = -strtoul (buf, NULL, 10) * PANGO_SCALE;
+			int rise = -strtod (buf, NULL) * PANGO_SCALE;
 			xmlFree (buf);
 			tag = new gccv::RiseTextTag (rise);
 		} else {
@@ -742,7 +771,7 @@ bool Text::LoadNode (xmlNodePtr node, unsigned &pos, int cur_size)
 	} else if (!strcmp ((const char*) node->name, "sup")) {
 		buf = (char*) xmlGetProp (node, (xmlChar*) "height");
 		if (buf) {
-			int rise = strtoul (buf, NULL, 10) * PANGO_SCALE;
+			int rise = strtod (buf, NULL) * PANGO_SCALE;
 			xmlFree (buf);
 			tag = new gccv::RiseTextTag (rise);
 		} else {
@@ -834,11 +863,19 @@ void Text::AddItem ()
 	View *view = doc->GetView ();
 	Theme *theme = doc->GetTheme ();
 	PangoFontDescription *desc = pango_font_description_new ();
-	pango_font_description_set_family (desc, doc->GetTextFontFamily ());
-	pango_font_description_set_style (desc, doc->GetTextFontStyle ());
-	pango_font_description_set_variant (desc, doc->GetTextFontVariant ());
-	pango_font_description_set_weight (desc, doc->GetTextFontWeight ());
-	pango_font_description_set_size (desc, doc->GetTextFontSize ());
+	if (m_GlobalTag == gccv::Invalid) {
+		pango_font_description_set_family (desc, doc->GetTextFontFamily ());
+		pango_font_description_set_style (desc, doc->GetTextFontStyle ());
+		pango_font_description_set_variant (desc, doc->GetTextFontVariant ());
+		pango_font_description_set_weight (desc, doc->GetTextFontWeight ());
+		pango_font_description_set_size (desc, doc->GetTextFontSize ());
+	} else {
+		pango_font_description_set_family (desc, theme->GetFontFamily ());
+		pango_font_description_set_style (desc, theme->GetFontStyle ());
+		pango_font_description_set_variant (desc, theme->GetFontVariant ());
+		pango_font_description_set_weight (desc, theme->GetFontWeight ());
+		pango_font_description_set_size (desc, theme->GetFontSize ());
+	}
 	if (m_ascent <= 0) {
 		PangoContext* pc = gccv::Text::GetContext ();
 		PangoLayout *layout = pango_layout_new (pc);
@@ -869,6 +906,17 @@ void Text::AddItem ()
 	text->SetInterline (m_Interline);
 	pango_font_description_free (desc);
 	text->SetText (m_buf.c_str ());
+	if (m_GlobalTag != gccv::Invalid && m_TagList.empty () && !m_buf.empty ()) {
+		// add the global tag
+		gccv::TextTag *tag = NULL;
+		if (m_GlobalTag == gcp::StoichiometryTag)
+			tag = new gcp::StoichiometryTextTag (static_cast < double > (theme->GetFontSize ()) / PANGO_SCALE);
+		if (tag) {
+			tag->SetStartIndex (0);
+			tag->SetEndIndex (m_buf.length ());
+			m_TagList.push_front (tag);
+		}
+	}
 	while (!m_TagList.empty ()) {
 		text->InsertTextTag (m_TagList.front ());
 		m_TagList.pop_front ();
@@ -878,6 +926,8 @@ void Text::AddItem ()
 
 void Text::UpdateItem ()
 {
+	if (!m_Item)
+		return;
 	Document *doc = static_cast <Document*> (GetDocument ());
 	Theme *theme = doc->GetTheme ();
 	reinterpret_cast <gccv::Text *> (m_Item)->SetPosition (m_x * theme->GetZoomFactor (), m_y * theme->GetZoomFactor ());
@@ -908,8 +958,10 @@ bool Text::OnChanged (bool save)
 
 void Text::SetSelected (int state)
 {
+	if (!m_Item)
+		return;
 	GOColor color;
-	switch (state) {	
+	switch (state) {
 	case SelStateUnselected:
 		color = 0;
 		break;
@@ -937,7 +989,7 @@ void Text::Transform2D (Matrix2D& m, double x, double y)
 	m_x -= m_length / 2 - x;
 	m_y -= m_height / 2 - m_ascent - y;
 }
-	
+
 double Text::GetYAlign ()
 {
 	return m_y;
