@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
  * theme.h
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2009 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,9 +27,6 @@
 
 #include <gcu/macros.h>
 #include <gcu/object.h>
-#include <glib/gtypes.h>
-#include <pango/pango.h>
-#include <goffice/app/go-conf.h>
 #include <libxml/tree.h>
 #include <list>
 #include <map>
@@ -88,6 +85,12 @@ The destructor.
 @return the theme name.
 */
 	std::string &GetName () {return m_Name;}
+
+/*!
+@return the theme name.
+*/
+	std::string &GetFileName () {return (m_FileName.length ())? m_FileName: m_Name;}
+
 /*!
 @param xml the xml document used for serialization.
 
@@ -125,9 +128,10 @@ Notify all the theme clients that at least one setting changed.
 	void NotifyChanged ();
 
 private:
-	std::string m_Name;
+	std::string m_Name, m_FileName;
 	std::set <gcu::Object*> m_Clients;
 	bool modified;
+	bool locked;
 
 /*!\fn GetBondLength()
 @return the theme default bond length in picometers.
@@ -162,7 +166,7 @@ GCU_RO_PROP (double, HashDist)
 */
 GCU_RO_PROP (double, StereoBondWidth)
 /*!\fn GetZoomFactor()
-@return the theme default scale used to convert real distance to canvas distance expressed in pixel per pm (at 100% zoom). 
+@return the theme default scale used to convert real distance to canvas distance expressed in pixel per pm (at 100% zoom).
 */
 GCU_RO_PROP (double, ZoomFactor)
 /*!\fn GetPadding()
@@ -288,13 +292,13 @@ The destructor.
 /*!
 @param name the name of the requested theme.
 
-@return the theme corresponding to \a name if any, or NULL. 
+@return the theme corresponding to \a name if any, or NULL.
 */
 	Theme *GetTheme (char const *name);
 /*!
 @param name the name of the requested theme.
 
-@return the theme corresponding to \a name if any, or NULL. 
+@return the theme corresponding to \a name if any, or NULL.
 */
 	Theme *GetTheme (std::string &name);
 /*!
