@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint
- * tools.h 
+ * tools.h
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,17 +25,22 @@
 #ifndef GCHEMPAINT_TOOLS_H
 #define GCHEMPAINT_TOOLS_H
 
-#include <gcu/dialog.h>
-#include <gcu/gcucomboperiodic.h>
+#include <gcugtk/dialog.h>
+#include <gcugtk/gcucomboperiodic.h>
 #include <map>
 
 /*!\file*/
+
+namespace gcugtk {
+class UIManager;
+}
+
 namespace gcp {
 
 /*!class Tools gcp/tools.h
 The GChemPaint tools box. Only one instance should be created.
 */
-class Tools: public gcu::Dialog
+class Tools: public gcugtk::Dialog
 {
 public:
 /*!
@@ -62,11 +67,11 @@ Addsa new toolbar.
 */
 	void AddToolbar (std::string &name);
 /*!
-@param manager a GtkUIManager.
+@param manager a gcu::UIManager.
 
-Sets the GtkUIManager associated to the user interface of the tools box.
+Sets the gcugtk::UIManager associated to the user interface of the tools box.
 */
-	void SetUIManager (GtkUIManager *manager);
+	void SetUIManager (gcugtk::UIManager *manager);
 /*!
 @param tool the Tool for which an options page has been added.
 @param i the rank of the page in the notebook.
@@ -108,8 +113,8 @@ Called by the framework when the Help button is clicked.
 	void OnHelp ();
 
 private:
-	GtkUIManager *m_UIManager;
-	GtkBox *m_ButtonsBox;
+	gcugtk::UIManager *m_UIManager;
+	GtkGrid *m_ButtonsGrid;
 	std::map<Tool*, int> m_Pages;
 	GtkNotebook *m_Book;
 	Tool *m_Tool;
