@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -293,6 +293,14 @@ Changes the selection state of \a object if it owns an item and of all its desce
 */
 	WidgetData *GetData () {return m_pData;}
 
+/*!
+@param x0 where to store the left horizontal coordinate of the visible area.
+@param y0 where to store the top vertical coordinate of the visible area.
+@param x1 where to store the right horizontal coordinate of the visible area.
+@param y1 where to store the bottom vertical coordinate of the visible area.
+
+Gets the visible area for the view.
+*/
 	void GetVisibleArea (double &x0, double &y0, double &x1, double &y1);
 
 	//Implementation
