@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint library
  * widgetdata.cc
  *
- * Copyright (C) 2002-20067 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -28,6 +28,7 @@
 #include "settings.h"
 #include "document.h"
 #include "application.h"
+#include "brackets.h"
 #include "operation.h"
 #include "theme.h"
 #include "tool.h"
@@ -36,6 +37,7 @@
 #include <gccv/item-client.h>
 #include <gccv/item.h>
 #include <cstring>
+#include <map>
 
 using namespace gcu;
 
@@ -212,20 +214,71 @@ void on_clear_data (GtkClipboard *clipboard, Object *obj)
 
 bool WidgetData::IsSelected (Object const *obj) const
 {
-	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
+	std::set < Object * >::const_iterator i, end = SelectedObjects.end ();
 	Object const *parent = obj->GetParent ();
 	if (parent && IsSelected (parent))
 		return true;
 	for (i = SelectedObjects.begin (); i != end; i++)
 		if (*i == obj)
 			return true;
-	
+
 	return false;
 }
 
+bool WidgetData::ChildrenSelected (gcu::Object const *obj) const
+{
+	if (!obj->HasChildren ())
+		return false;
+	std::map <std::string, gcu::Object *>::const_iterator i;
+	std::set < Object * >::const_iterator j,  end = SelectedObjects.end ();
+	for (Object const *child = obj->GetFirstChild (i); child; child = obj->GetNextChild (i)) {
+		if (child->GetType () == BracketsType) // do not consider brackets,here
+			continue;
+		for (j = SelectedObjects.begin (); j != end; j++)
+			if (*j == child)
+				break;
+		if (j == end && !ChildrenSelected (child))
+		    return false;
+	}
+	return true;
+}
+
+gcu::Object *WidgetData::GetSelectedAncestor (gcu::Object *child)
+{
+	Object *parent = child->GetParent ();
+	if (parent->GetType () == DocumentType)
+		return NULL;
+	Object *ancestor = GetSelectedAncestor (parent);
+	if (ancestor)
+		return ancestor;
+	// check if the object needs a specific parent and return NULL if so
+	if (!parent->GetDocument ()->GetApplication ()->GetRules (parent->GetType (), RuleMustBeIn).empty ())
+		return NULL;
+	return (ChildrenSelected (parent))? parent: NULL;
+}
+
+void WidgetData::SimplifySelection ()
+{
+	std::set < Object * >::iterator i, end = SelectedObjects.end ();
+	std::set < Object * > RealSelection;
+	Object *parent;
+	gcu::Application *app = m_View->GetDoc ()->GetApplication ();
+	for (i = SelectedObjects.begin (); i != end; i++) {
+		parent = GetSelectedAncestor (*i);
+		if (parent)
+			RealSelection.insert (parent);
+		else if (app->GetRules ((*i)->GetType (), RuleMustBeIn).empty ())
+			RealSelection.insert (*i);
+	}
+	UnselectAll ();
+	std::set <Object *>::iterator j, jend = RealSelection.end ();
+	for (j = RealSelection.begin (); j != jend; j++)
+		SetSelected (*j);
+}
+
 void WidgetData::Unselect (Object *obj)
 {
-	SelectedObjects.remove (obj);
+	SelectedObjects.erase (obj);
 	m_View->SetSelectionState (obj, SelStateUnselected);
 }
 
@@ -233,8 +286,8 @@ void WidgetData::UnselectAll ()
 {
 	Object* obj;
 	while (!SelectedObjects.empty ()) {
-		obj = SelectedObjects.front ();
-		SelectedObjects.pop_front ();
+		obj = *SelectedObjects.begin ();
+		SelectedObjects.erase (obj);
 		Unselect (obj);
 	}
 }
@@ -242,14 +295,14 @@ void WidgetData::UnselectAll ()
 void WidgetData::SetSelected (Object *obj, int state)
 {
 	if (!IsSelected (obj)) {
-		SelectedObjects.push_front (obj);
+		SelectedObjects.insert (obj);
 		m_View->SetSelectionState (obj, state);
 	}
 }
 
 void WidgetData::MoveSelectedItems (double dx, double dy)
 {
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
+	std::set < Object * >::iterator i, end = SelectedObjects.end ();
 	for (i = SelectedObjects.begin (); i != end; i++)
 		MoveItems (*i, dx, dy);
 }
@@ -273,7 +326,7 @@ void WidgetData::MoveSelection (double dx, double dy)
 {
 	if (!SelectedObjects.size ())
 		return;
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
+	std::set < Object * >::iterator i, end = SelectedObjects.end ();
 	Document* pDoc = m_View->GetDoc ();
 	Operation* pOp = pDoc-> GetNewOperation (GCP_MODIFY_OPERATION);
 	Theme *pTheme = pDoc->GetTheme ();
@@ -289,11 +342,11 @@ void WidgetData::MoveSelection (double dx, double dy)
 void WidgetData::RotateSelection (double dx, double dy, double angle)
 {
 	Theme *pTheme = m_View->GetDoc ()->GetTheme ();
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
+	std::set < Object * >::iterator i, end = SelectedObjects.end ();
 	Matrix2D m (angle);
 	for (i = SelectedObjects.begin (); i != end; i++) {
 		(*i)->Transform2D (m, dx / pTheme->GetZoomFactor (), dy / pTheme->GetZoomFactor ());
-		m_View->Update(*i);
+		m_View->Update (*i);
 	}
 }
 
@@ -312,7 +365,7 @@ void WidgetData::Copy (GtkClipboard* clipboard)
 	xmlNsPtr ns = xmlNewNs ((*pDoc)->children, (xmlChar*) "http://www.nongnu.org/gchempaint", (xmlChar*) "gcp");
 	xmlSetNs ((*pDoc)->children, ns);
 //FIXME: implement exception handling
-	std::list<Object*>::iterator i, end= SelectedObjects.end ();
+	std::set < Object * >::iterator i, end= SelectedObjects.end ();
 	xmlNodePtr child;
 	for (i = SelectedObjects.begin(); i != end; i++)
 		if ((child = (*i)->Save (pXmlDoc)))
@@ -329,8 +382,9 @@ void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect &rect) const
 	double x1, y1, x2,  y2;
 	client = dynamic_cast <gccv::ItemClient const *> (obj);
 	gccv::Item const *item;
-	if (client && (item = client->GetItem ()) && item->GetParent ()->GetParent () == NULL) {
-		client->GetItem ()->GetBounds (x1, y1, x2, y2);
+	// will not work if the object item is not top level, but can this happen?
+	if (client && (item = client->GetItem ()) && item->IsTopLevel ()) {
+		item->GetBounds (x1, y1, x2, y2);
 		if (x2 > 0.) {
 			if (!go_finite (rect.x0)) {
 				rect.x0 = x1;
@@ -357,7 +411,7 @@ void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect &rect) const
 
 void WidgetData::GetSelectionBounds (gccv::Rect &rect) const
 {
-	std::list<Object*>::const_iterator i, end = SelectedObjects.end ();
+	std::set < Object * >::const_iterator i, end = SelectedObjects.end ();
 	rect.x0 = go_nan;
 	for (i = SelectedObjects.begin (); i != end; i++)
 		GetObjectBounds (*i, rect);
@@ -373,6 +427,26 @@ void WidgetData::GetObjectBounds (Object const *obj, gccv::Rect *rect) const
 		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
 }
 
+void WidgetData::GetObjectsBounds (std::set <gcu::Object const *> const &objects, gccv::Rect *rect) const
+{
+	rect->x0 = go_nan;
+	std::set <gcu::Object const *>::iterator it, end = objects.end ();
+	for (it = objects.begin (); it != end; it++)
+		GetObjectBounds (*it, *rect);
+	if (!go_finite (rect->x0))
+		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
+}
+
+void WidgetData::GetObjectsBounds (std::set <gcu::Object *> const &objects, gccv::Rect *rect) const
+{
+	rect->x0 = go_nan;
+	std::set <gcu::Object *>::iterator it, end = objects.end ();
+	for (it = objects.begin (); it != end; it++)
+		GetObjectBounds (*it, *rect);
+	if (!go_finite (rect->x0))
+		rect->x0 = rect->y0 = rect->x1 = rect->y1 = 0.;
+}
+
 void WidgetData::SelectAll ()
 {
 	std::list <gccv::Item *>::iterator it;
@@ -400,7 +474,7 @@ xmlDocPtr WidgetData::GetXmlDoc (GtkClipboard* clipboard)
 
 void WidgetData::ShowSelection (bool state)
 {
-	std::list<Object*>::iterator i, end = SelectedObjects.end ();
+	std::set < Object * >::iterator i, end = SelectedObjects.end ();
 	for (i = SelectedObjects.begin (); i != end; i++)
 		m_View->SetSelectionState (*i, (state)? SelStateSelected: SelStateUnselected);
 }
