@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Chemisty Utils
- * gcr/document.cc 
+ * gcr/document.cc
  *
  * Copyright (C) 2002-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,7 +25,15 @@
 #include "config.h"
 #include <cstring>
 #include "document.h"
+#include "application.h"
 #include "view.h"
+#include "celldlg.h"
+#include "atomsdlg.h"
+#include "linesdlg.h"
+#include "sizedlg.h"
+#include "cleavagesdlg.h"
+#include "window.h"
+#include <gcugtk/filechooser.h>
 #include <gcu/cylinder.h>
 #include <gcu/matrix.h>
 #include <gcu/objprops.h>
@@ -42,6 +50,7 @@
 #include <vector>
 #include <GL/gl.h>
 #include <cstring>
+#include <iostream>
 #include <sstream>
 
 #define __max(x,y)  ((x) > (y)) ? (x) : (y)
@@ -50,7 +59,12 @@
 using namespace std;
 
 namespace gcr {
-	
+
+#define SAVE	1
+#define LOAD	2
+#define XML		3
+#define FORMAT	4
+
 gchar const *LatticeName[] = {
 	"simple cubic",
 	"body-centered cubic",
@@ -73,34 +87,14 @@ Document::Document (gcu::Application *App): gcu::GLDocument (App),
 {
 	m_xmin = m_ymin = m_zmin = 0;
 	m_xmax = m_ymax = m_zmax = 1;
+	Init ();
 }
 
 
 Document::~Document()
 {
-	while (!AtomDef.empty ()) {
-		delete AtomDef.front ();
-		AtomDef.pop_front ();
-	}
-	while (!Atoms.empty ()) {
-		delete Atoms.front ();
-		Atoms.pop_front ();
-	}
-	while (!LineDef.empty ()) {
-		delete LineDef.front ();
-		LineDef.pop_front ();
-	}
-	while (!Lines.empty ()) {
-		delete Lines.front ();
-		Lines.pop_front ();
-	}
-	while (!Cleavages.empty ()) {
-		delete Cleavages.front ();
-		Cleavages.pop_front ();
-	}
-	while (!m_Views.empty ()) {
-		m_Views.pop_back ();
-	}
+	g_free (m_filename);
+	Reinit ();
 }
 
 void Document::Reinit()
@@ -126,6 +120,9 @@ void Document::Reinit()
 		delete Cleavages.front ();
 		Cleavages.pop_front ();
 	}
+	g_free (m_Author);
+	g_free (m_Mail);
+	g_free (m_Comment);
 	Init ();
 }
 
@@ -140,11 +137,11 @@ void Document::Init()
 	m_xmax = m_ymax = m_zmax = 1;
 	m_FixedSize = false;
 	m_MaxDist = 0;
-	if (m_Views.size() == 0)
-	{
-		View* pView = CreateNewView();
-		m_Views.push_back(pView);
-	}
+	m_filename = NULL;
+	m_Label = NULL;
+	m_Author = NULL;
+	m_Mail = NULL;
+	m_Comment = NULL;
 }
 
 void Document::ParseXMLTree (xmlNode* xml)
@@ -156,19 +153,6 @@ void Document::ParseXMLTree (xmlNode* xml)
 	Reinit ();
 	m_SpaceGroup = NULL;
 	//look for generator node
-	unsigned version = 0xffffff , major, minor, micro;
-	node = xml->children;
-	while (node) {
-		if (!strcmp ((const char*)(node->name), "generator"))
-			break;
-		node = node->next;
-	}
-	if (node) {
-		txt = (char*) xmlNodeGetContent (node);
-		if (sscanf(txt, "Gnome Crystal %d.%d.%d", &major, &minor, &micro) == 3)
-			version = micro + minor * 0x100 + major * 0x10000;
-		xmlFree (txt);
-	}
 	node = xml->children;
 	while(node) {
 		if (!strcmp ((gchar*) node->name, "text"));
@@ -225,8 +209,10 @@ void Document::ParseXMLTree (xmlNode* xml)
 		} else if (!strcmp ((gchar*) node->name, "atom")) {
 			Atom *pAtom = CreateNewAtom ();
 			AddChild (pAtom);
-			if (!pAtom->Load (node))
+			if (!pAtom->Load (node)) {
+				AtomDef.remove (pAtom);
 				delete pAtom;
+			}
 		} else if (!strcmp ((gchar*) node->name, "line")) {
 			Line *pLine = CreateNewLine ();
 			if (pLine->Load (node))
@@ -240,11 +226,11 @@ void Document::ParseXMLTree (xmlNode* xml)
 			else
 				delete pCleavage;
 		} else if (!strcmp ((gchar*) node->name, "view")) {
-			if (!bViewLoaded) {
-				m_Views.front ()->Load (node); //the first view is created with the document
-				bViewLoaded = true;
-			} else
+			if (!bViewLoaded &&! m_Views.empty ())
+				m_Views.front ()->Load (node);
+			else
 				LoadNewView (node);
+			bViewLoaded = true;
 		}
 		node = node->next;
 	}
@@ -252,40 +238,42 @@ void Document::ParseXMLTree (xmlNode* xml)
 	Update ();
 }
 
-bool Document::LoadNewView(G_GNUC_UNUSED xmlNodePtr node)
+bool Document::LoadNewView (G_GNUC_UNUSED xmlNodePtr node)
 {
-	return true;
+	Window *pWindow = static_cast < Application * > (GetApplication ())->CreateNewWindow (this);
+	View *pView = pWindow->GetView ();
+	bool result = pView->Load (node);
+	if (!result)
+		delete pWindow;
+	return result;
 }
 
-View *Document::GetView()
+View *Document::GetView ()
 {
-	if (m_Views.size() == 0)
-	{
-		View* pView = CreateNewView();
-		m_Views.push_back(pView);
+	if (m_Views.size() == 0) {
+		View* pView = CreateNewView ();
+		m_Views.push_back (pView);
 	}
-	return m_Views.front();
+ 	return m_Views.front ();
 }
 
-void Document::Update()
+void Document::Update ()
 {
-	m_Empty = (AtomDef.empty() && LineDef.empty()) ? true : false;
+	m_Empty = AtomDef.empty () && LineDef.empty ();
 	Atom atom;
 	Line line;
 	gdouble alpha = m_alpha * M_PI / 180;
 	gdouble beta = m_beta * M_PI / 180;
 	gdouble gamma = m_gamma * M_PI / 180;
 	// Remove all atoms
-	while (!Atoms.empty())
-	{
-		delete Atoms.front();
-		Atoms.pop_front();
+	while (!Atoms.empty ()) {
+		delete Atoms.front ();
+		Atoms.pop_front ();
 	}
 	// Remove all bonds
-	while (!Lines.empty())
-	{
-		delete Lines.front();
-		Lines.pop_front();
+	while (!Lines.empty ()) {
+		delete Lines.front ();
+		Lines.pop_front ();
 	}
 
 	// update space group
@@ -338,7 +326,7 @@ void Document::Update()
 			break;
 		}
 	}
-	
+
 	////////////////////////////////////////////////////////////
 	//Establish list of lines
 	LineList::iterator j, jend = LineDef.end();
@@ -418,6 +406,9 @@ void Document::Update()
 		std::vector<double>::iterator m;
 		unsigned n;
 
+		// we might have invalid cleavages, so we need to skip them
+		if ((*k)->Planes () == 0 || ((*k)->h () == 0 && (*k)->k () == 0 && (*k)->l () == 0))
+			continue;	// invalid cleavage
 		//scalar products calculus and storing
 		for (i = Atoms.begin(); i != Atoms.end(); i++)
 		{
@@ -450,7 +441,7 @@ void Document::Update()
 
 		ScalarProducts.clear() ;
 	}
-	
+
 	//Transform coordinates to Cartesians and find center of visible view
 	gdouble x, y, z, d,
 		xmin = G_MAXDOUBLE, ymin = G_MAXDOUBLE, zmin = G_MAXDOUBLE,
@@ -516,6 +507,18 @@ void Document::Update()
 		(*j)->Move (- x, - y, - z);
 	}
 	m_SpaceGroup = FindSpaceGroup ();
+	UpdateAllViews ();
+}
+
+void Document::UpdateAllViews()
+{
+	std::list < View * >::iterator i;
+	for (i = m_Views.begin (); i != m_Views.end (); i++) {
+		(*i)->Update();
+		Window *window = (*i)->GetWindow ();
+		if (window)
+			window->ClearStatus ();
+	}
 }
 
 void Document::Duplicate (Atom& atom)
@@ -555,7 +558,7 @@ void Document::Duplicate (Line& line)
 		LineX.Move (1,0,0) ;
 	}
 }
-	
+
 void Document::Draw (gcu::Matrix const &m) const
 {
 	gcu::Vector v, v1;
@@ -623,9 +626,23 @@ xmlDocPtr Document::BuildXMLTree () const
 	xmlNodePtr node;
 	xmlNsPtr ns;
 
+	// first check if dialogs are open and data coherent
+	gcu::Dialog *dlg;
+	if ((dlg = GetDialog ("atoms"))) {
+		const_cast <Document * > (this)->CheckAtoms ();
+		static_cast < AtomsDlg * > (dlg)->ReloadData ();
+	}
+	if ((dlg = GetDialog ("cleavages"))) {
+		const_cast <Document * > (this)->CheckCleavages ();
+		static_cast < CleavagesDlg * > (dlg)->ReloadData ();
+	}
+	if ((dlg = GetDialog ("lines"))) {
+		const_cast <Document * > (this)->CheckLines ();
+		static_cast < LinesDlg * > (dlg)->ReloadData ();
+	}
 	xml = xmlNewDoc((xmlChar*)"1.0");
 	if (xml == NULL) {throw(1);}
-	
+
 	xmlDocSetRootElement (xml,  xmlNewDocNode(xml, NULL, (xmlChar*)"crystal", NULL));
 	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gcrystal", (xmlChar*) "gcry");
 	xmlSetNs (xml->children, ns);
@@ -634,7 +651,7 @@ xmlDocPtr Document::BuildXMLTree () const
 	{
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"generator", (xmlChar*)GetProgramId());
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
-		
+
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"lattice", (xmlChar*)LatticeName[m_lattice]);
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		if (m_SpaceGroup) {
@@ -674,42 +691,42 @@ xmlDocPtr Document::BuildXMLTree () const
 		gcu::WriteFloat (node, "alpha", m_alpha);
 		gcu::WriteFloat (node, "beta", m_beta);
 		gcu::WriteFloat (node, "gamma", m_gamma);
-	
+
 		node = xmlNewDocNode(xml, NULL, (xmlChar*)"size", NULL);
 		if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		gcu::WritePosition(xml, node, "start", m_xmin, m_ymin, m_zmin);
 		gcu::WritePosition(xml, node, "end", m_xmax, m_ymax, m_zmax);
 		if (m_FixedSize)
 			xmlNewProp (node, (xmlChar *) "fixed", (xmlChar *) "true");
-		
+
 		AtomList::const_iterator i;
 		for (i = AtomDef.begin(); i != AtomDef.end(); i++)
 		{
 			node = (*i)->Save(xml);
 			if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		}
-	
+
 		LineList::const_iterator j;
 		for (j = LineDef.begin(); j != LineDef.end(); j++)
 		{
 			node = (*j)->Save(xml);
 			if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		}
-	
+
 		CleavageList::const_iterator k;
 		for (k = Cleavages.begin(); k != Cleavages.end(); k++)
 		{
 			node = (*k)->Save(xml);
 			if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		}
-		
+
 		list<View*>::const_iterator view;
 		for (view = m_Views.begin(); view != m_Views.end(); view++)
 		{
 			node = (*view)->Save(xml);
 			if (node) xmlAddChild(xml->children, node); else throw (int) 0;
 		}
-	
+
 		return xml;
 	}
 	catch (int num)
@@ -816,22 +833,22 @@ bool Document::SetProperty (unsigned property, char const *value)
 {
 	switch (property) {
 	case GCU_PROP_CELL_A:
-		m_a = strtod (value, NULL) * GetScale ();
+		m_a = g_ascii_strtod (value, NULL) * GetScale ();
 		break;
 	case GCU_PROP_CELL_B:
-		m_b = strtod (value, NULL) * GetScale ();
+		m_b = g_ascii_strtod (value, NULL) * GetScale ();
 		break;
 	case GCU_PROP_CELL_C:
-		m_c = strtod (value, NULL) * GetScale ();
+		m_c = g_ascii_strtod (value, NULL) * GetScale ();
 		break;
 	case GCU_PROP_CELL_ALPHA:
-		m_alpha = strtod (value, NULL);
+		m_alpha = g_ascii_strtod (value, NULL);
 		break;
 	case GCU_PROP_CELL_BETA:
-		m_beta = strtod (value, NULL);
+		m_beta = g_ascii_strtod (value, NULL);
 		break;
 	case GCU_PROP_CELL_GAMMA:
-		m_gamma = strtod (value, NULL);
+		m_gamma = g_ascii_strtod (value, NULL);
 		break;
 	case GCU_PROP_CHEMICAL_NAME_COMMON:
 		m_NameCommon = value;
@@ -890,6 +907,14 @@ bool Document::SetProperty (unsigned property, char const *value)
 		}
 		break;
 	}
+	case GCU_PROP_DOC_CREATOR:
+		g_free (m_Author);
+		m_Author = g_strdup (value);
+		break;
+	case GCU_PROP_DOC_CREATOR_EMAIL:
+		g_free (m_Mail);
+		m_Mail = g_strdup (value);
+		break;
 	default:
 		return false;
 	}
@@ -928,6 +953,15 @@ std::string Document::GetProperty (unsigned property) const
 		break;
 	case GCU_PROP_SPACE_GROUP:
 		return m_SpaceGroup->GetHallName ();
+	case GCU_PROP_DOC_CREATOR:
+		if (m_Author)
+				 res << m_Author;
+		break;
+	case GCU_PROP_DOC_CREATOR_EMAIL:
+		if (m_Mail)
+				res << m_Mail;
+		g_free (m_Mail);
+		break;
 	default:
 		return GLDocument::GetProperty (property);
 	}
@@ -1093,6 +1127,7 @@ end_loop:;
 		a = new Atom (**i);
 		v = a->GetVector ();
 		std::list <gcu::Vector> vv = res->Transform (v);
+		jend = vv.end ();
 		for (j = vv.begin (); j != jend; j++) {
 			x = (*j).GetX ();
 			y = (*j).GetY ();
@@ -1168,4 +1203,623 @@ void Document::SetCell (Lattice lattice, double a, double b, double c, double al
 	m_gamma = gamma;
 }
 
+static int gcd_euler (int n1, int n2)
+{
+	int buf;
+	if (n1 < n2) {
+		buf = n1;
+		n1 = n2;
+		n2 = buf;
+	}
+	while (1) {
+		if (n2 == 0)
+			return n1;
+		buf = n1 % n2;
+		n1 = n2;
+		n2 = buf;
+	}
+}
+
+void Document::CheckCleavages ()
+{
+	std::set < Cleavage * > garbage;
+	CleavageList::iterator i, j, end = Cleavages.end ();
+	int gcd;
+	for (i = Cleavages.begin (); i != end; i++) {
+		if ((*i)->Planes () == 0) {
+			garbage.insert (*i);
+			continue;
+		}
+		// now divide h, k, and l by their gcd, using Euclid's algorithm
+		// since we probably don't need better there
+		gcd = abs ((*i)->h ());
+		if (!gcd) {
+			gcd = abs ((*i)->k ());
+			if (!gcd) {
+				gcd = abs ((*i)->l ());
+				if (!gcd) {
+					garbage.insert (*i);
+					continue;
+				}
+			}
+		} else
+			gcd = gcd_euler (gcd, abs ((*i)->k ()));
+		gcd = gcd_euler (gcd, abs ((*i)->l ()));
+		(*i)->h () /= gcd;
+		(*i)->k () /= gcd;
+		(*i)->l () /= gcd;
+		// now we look for equivalent cleavages in the list beginning
+		for (j = Cleavages.begin (); j != i; j++)
+			if (*i == *j) {
+				if ((*j)->Planes () > (*i)->Planes ()) // use the largest planes number
+					(*i)->Planes () = (*j)->Planes ();
+				garbage.insert (*j);
+				break;
+			}
+	}
+	std::set < Cleavage * >::iterator k,kend = garbage.end ();
+	for (k = garbage.begin (); k != kend; k++) {
+		Cleavages.remove (*k);
+		delete *k;
+	}
+}
+
+void Document::CheckAtoms ()
+{
+	std::set < Atom * > garbage;
+	AtomList::iterator i, j, end = AtomDef.end ();
+	for (i = AtomDef.begin (); i != end; i++) {
+		for (j = AtomDef.begin (); j != i; j++)
+			if (*i == *j) {
+				garbage.insert (*j);
+				break;
+			}
+	}
+	std::set < Atom * >::iterator k,kend = garbage.end ();
+	for (k = garbage.begin (); k != kend; k++) {
+		AtomDef.remove (*k);
+		delete *k;
+	}
+}
+
+void Document::CheckLines ()
+{
+	std::set < Line * > garbage;
+	LineList::iterator i, j, end = LineDef.end ();
+	for (i = LineDef.begin (); i != end; i++) {
+		for (j = LineDef.begin (); j != i; j++)
+			if (*i == *j) {
+				garbage.insert (*j);
+				break;
+			}
+	}
+	std::set < Line * >::iterator k,kend = garbage.end ();
+	for (k = garbage.begin (); k != kend; k++) {
+		LineDef.remove (*k);
+		delete *k;
+	}
+}
+
+void Document::Define (unsigned nPage)
+{
+	gcu::Dialog *dialog;
+	switch (nPage) {
+	case 0:
+		dialog = GetDialog ("cell");
+		if (dialog)
+			dialog->Present ();
+		else
+			new gcr::CellDlg (static_cast < Application * > (m_App), this);
+		break;
+	case 1:
+		dialog = GetDialog ("atoms");
+		if (dialog)
+			dialog->Present ();
+		else
+			new gcr::AtomsDlg (static_cast <Application *> (m_App), this);
+		break;
+	case 2:
+		dialog = GetDialog ("lines");
+		if (dialog)
+			dialog->Present ();
+		else
+			new gcr::LinesDlg (static_cast <Application *> (m_App), this);
+		break;
+	case 3:
+		dialog = GetDialog ("size");
+		if (dialog)
+			dialog->Present ();
+		else
+			new gcr::SizeDlg (static_cast <Application *> (m_App), this);
+		break;
+	case 4:
+		dialog = GetDialog ("cleavages");
+		if (dialog)
+			dialog->Present ();
+		else
+			new gcr::CleavagesDlg (static_cast < Application * > (m_App), this);
+		break;
+	}
+}
+void Document::AddView(View* pView)
+{
+	m_Views.push_back (pView);
+	RenameViews ();
+	if (!GetEmpty ())
+		SetDirty (true);
+}
+
+bool Document::RemoveView (View* pView)
+{
+	if (m_Views.size () > 1) {
+		m_Views.remove (pView);
+		RenameViews ();
+		if (!m_bClosing && !GetEmpty ())
+			SetDirty (true);
+		return true;
+	}
+	if (GetDirty ()) {
+		if (!VerifySaved ())
+			return false;
+	}
+	delete this;
+	return true;
+}
+
+void Document::RemoveAllViews ()
+{
+	while (m_Views.size () > 1)
+		m_Views.front ()->GetWindow ()->Destroy ();
+	// The last one is deleted separately since this will be destroyed !
+	m_Views.front ()->GetWindow ()->Destroy ();
+}
+
+bool Document::VerifySaved()
+{
+	m_bClosing = true;
+	if (!GetDirty ())
+		return true;
+	GtkWidget* mbox;
+	int res;
+	do
+	{
+		mbox = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _("\"%s\" has been modified.  Do you wish to save it?"), GetLabel ());
+		gtk_dialog_add_button(GTK_DIALOG(mbox),  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+		res = gtk_dialog_run(GTK_DIALOG(mbox));
+		gtk_widget_destroy(mbox);
+		if (res == GTK_RESPONSE_YES) {
+			if (m_filename == NULL) {
+				list<string> l;
+				l.push_front ("application/x-gcrystal");
+				gcugtk::FileChooser (static_cast < gcugtk::Application * > (m_App), true, l, this);
+			}
+			if (m_filename)
+				Save ();
+		}
+	}
+	while ((res == GTK_RESPONSE_YES) && (m_filename == NULL));
+	if (res == GTK_RESPONSE_NO)
+		SetDirty (false);
+	else if (res == GTK_RESPONSE_CANCEL) m_bClosing = false;
+	return (res != GTK_RESPONSE_CANCEL);
+}
+
+void Document::RenameViews ()
+{
+	list <View *>::iterator i, iend = m_Views.end ();
+	int n = 1, max = m_Views.size ();
+	for (i = m_Views.begin (); i != iend; i++) {
+		Window *window = (*i)->GetWindow ();
+		GtkWindow *w = window->GetWindow ();
+		if (!w)
+			continue;
+		if (max > 1) {
+			char *t = g_strdup_printf ("%s (%i)", m_Label, n++);
+			gtk_window_set_title (w, t);
+			g_free (t);
+		} else
+			gtk_window_set_title (w, GetLabel ());
+		window->ActivateActionWidget ("ui/MainMenu/FileMenu/Save", !m_ReadOnly);
+		window->ActivateActionWidget ("ui/MainToolbar/Save", !m_ReadOnly);
+	}
+}
+
+void Document::SetFileName (const string &filename)
+{
+	GFile *file = g_file_new_for_uri (filename.c_str ());
+	GError *error = NULL;
+	if (g_file_query_exists (file, NULL)) {
+		GFileInfo *info = g_file_query_info (file,
+								  G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE,
+								  G_FILE_QUERY_INFO_NONE, NULL, &error);
+		if (error) {
+			g_warning ("GIO error: %s", error->message);
+			g_error_free (error);
+			m_ReadOnly = true;
+		} else
+			m_ReadOnly = !g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+		if (info)
+			g_object_unref (info);
+	} else
+		m_ReadOnly = false;
+	g_object_unref (file);
+	if (m_filename)
+		g_free (m_filename);
+	m_filename = g_strdup (filename.c_str ());
+	char *dirname = g_path_get_dirname (m_filename);
+	m_App->SetCurDir (dirname);
+	g_free (dirname);
+	int i = filename.length () - 1;
+	while ((m_filename[i] != '/') && (i >= 0))
+		i--;
+	i++;
+	int j = filename.length () - 1;
+	while ((i < j) && (m_filename[j] != '.'))
+		j--;
+	char *buf = (strcmp (m_filename + j, ".gcrystal"))? g_strdup (m_filename + i): g_strndup (m_filename + i, j - i);
+	char *unescaped = g_uri_unescape_string (buf, NULL);
+	g_free (buf);
+	m_DefaultLabel = unescaped;
+	if (m_Title.length () == 0) {
+		g_free (m_Label);
+		m_Label = unescaped;
+	} else
+		g_free (unescaped);
+}
+
+void Document::SetTitle(const gchar* title)
+{
+	m_Title = title? title: "";
+	g_free (m_Label);
+	m_Label = title? g_strdup (title): NULL;
+}
+
+void Document::SetTitle (std::string& title)
+{
+	m_Title = title;
+	g_free (m_Label);
+	m_Label = title.length ()? g_strdup (title.c_str ()): NULL;
+}
+
+void Document::SetAuthor (char const *author)
+{
+	g_free (m_Author);
+	m_Author = g_strdup (author);
+}
+
+void Document::SetMail (char const *mail)
+{
+	g_free (m_Mail);
+	m_Mail = g_strdup (mail);
+}
+
+void Document::SetComment (char const *comment)
+{
+	g_free (m_Comment);
+	m_Comment = g_strdup (comment);
+}
+
+void Document::SetLabel (char const *label)
+{
+	g_free (m_Label);
+	m_Label = g_strdup (label);
+}
+
+typedef struct {int n; std::list<gcr::Atom*> l;} sAtom;
+typedef struct {int n; std::list<gcr::Line*> l;} sLine;
+
+void Document::OnExportVRML (const string &FileName) const
+{
+	char tmp[128];
+	double x0, x1, x2, x3, x4, x5;
+	int n = 0;
+	try {
+		ostringstream file;
+		GError *error = NULL;
+		GFile *stream = g_file_new_for_uri (FileName.c_str ());
+		GOutputStream *output = G_OUTPUT_STREAM (g_file_create (stream, G_FILE_CREATE_NONE, NULL, &error));
+		if (error) {
+			cerr << "gio error: " << error->message << endl;
+			g_error_free (error);
+			g_object_unref (file);
+			throw (int) 1;
+		}
+		std::map<std::string, sAtom>AtomsMap;
+		std::map<std::string, sLine>LinesMap;
+
+		file << "#VRML V2.0 utf8" << endl;
+
+		//Create prototypes for atoms
+		gcr::AtomList::const_iterator i;
+		for (i = Atoms.begin(); i != Atoms.end(); i++)
+		{
+			(*i)->GetColor(&x0, &x1, &x2, &x3);
+			snprintf(tmp, sizeof(tmp), "%g %g %g %g %g", (*i)->GetSize(), x0, x1, x2, x3);
+			if (AtomsMap[tmp].l.empty())
+			{
+				AtomsMap[tmp].n = n;
+				file << "PROTO Atom" << n++ << " [] {Shape {" << endl << "\tgeometry Sphere {radius " << (*i)->GetSize() / 100 << "}" << endl;
+				file << "\tappearance Appearance {" << endl << "\t\tmaterial Material {" << endl << "\t\t\tdiffuseColor " << x0 << " " << x1 << " " << x2 << endl;
+				if (x3 < 1) file << "\t\t\ttransparency " << (1 - x3) << endl;
+				file << "\t\t\tspecularColor 1 1 1" << endl << "\t\t\tshininess 0.9" << endl << "\t\t}" << endl << "\t}\r\n}}" << endl;
+			}
+			AtomsMap[tmp].l.push_back(*i);
+		}
+
+		//Create prototypes for bonds
+		gcr::LineList::const_iterator j;
+		n = 0;
+		for (j = Lines.begin(); j != Lines.end(); j++)
+		{
+			(*j)->GetColor(&x0, &x1, &x2, &x3);
+			snprintf(tmp, sizeof(tmp), "%g %g %g %g %g %g", (*j)->Long(), (*j)->GetRadius(), x0, x1, x2, x3);
+			if (LinesMap[tmp].l.empty())
+			{
+				LinesMap[tmp].n = n;
+				file << "PROTO Bond" << n++ << " [] {Shape {" << endl << "\tgeometry Cylinder {radius " << (*j)->GetRadius() / 100 << "\theight " << (*j)->Long() / 100 << "}" << endl;
+				file << "\tappearance Appearance {" << endl << "\t\tmaterial Material {" << endl << "\t\t\tdiffuseColor " << x0 << " " << x1 << " " << x2 << endl;
+				if (x3 < 1) file << "\t\t\ttransparency " << (1 - x3) << endl;
+				file << "\t\t\tspecularColor 1 1 1" << endl << "\t\t\tshininess 0.9" << endl << "\t\t}" << endl << "\t}\r\n}}" << endl;
+			}
+			LinesMap[tmp].l.push_back(*j);
+		}
+
+		//world begin
+		m_pActiveView->GetBackgroundColor(&x0, &x1, &x2, &x3);
+		file << "Background{skyColor " << x0 << " " << x1 << " " << x2 << "}" << endl;
+		file << "Viewpoint {fieldOfView " << m_pActiveView->GetFoV()/90*1.570796326794897 <<"\tposition 0 0 " << m_pActiveView->GetPos() / 100 << "}" << endl;
+		m_pActiveView->GetRotation(&x0, &x1, &x2);
+		gcu::Matrix m(x0/90*1.570796326794897, x1/90*1.570796326794897, x2/90*1.570796326794897, gcu::euler);
+		file << "Transform {" << endl << "\tchildren [" << endl;
+
+		std::map<std::string, sAtom>::iterator k;
+		for (k = AtomsMap.begin(); k != AtomsMap.end(); k++)
+		{
+			for (i = (*k).second.l.begin(); i != (*k).second.l.end(); i++)
+			{
+				if (!(*i)->IsCleaved())
+				{
+					x0 = (*i)->x();
+					x1 = (*i)->y();
+					x2 = (*i)->z();
+					m.Transform(x0, x1, x2);
+					file << "\t\tTransform {translation " << x1 / 100 << " " << x2 / 100 << " " << x0 / 100\
+						<<  " children [Atom" << (*k).second.n << " {}]}" << endl;
+				}
+			}
+			(*k).second.l.clear();
+		}
+		AtomsMap.clear();
+
+		std::map<std::string, sLine>::iterator l;
+		n = 0;
+		for (l = LinesMap.begin(); l != LinesMap.end(); l++)
+		{
+			for (j = (*l).second.l.begin(); j != (*l).second.l.end(); j++)
+			{
+				if (!(*j)->IsCleaved())
+				{
+					x0 = (*j)->X1();
+					x1 = (*j)->Y1();
+					x2 = (*j)->Z1();
+					m.Transform(x0, x1, x2);
+					x3 = (*j)->X2();
+					x4 = (*j)->Y2();
+					x5 = (*j)->Z2();
+					m.Transform(x3, x4, x5);
+					gcr::Line line(gcr::unique, x0, x1, x2, x3, x4, x5, 0.0, 0.0, 0.0, 0.0, 0.0);
+					line.GetRotation(x0, x1, x2, x3);
+					file << "\t\tTransform {" << endl << "\t\t\trotation " << x1 << " " << x2 << " " << x0 << " " << x3 << endl;
+					x0 = (line.X1() + line.X2()) / 200;
+					x1 = (line.Y1() + line.Y2()) / 200;
+					x2 = (line.Z1() + line.Z2()) / 200;
+					file << "\t\t\ttranslation " << x1 << " " << x2  << " " << x0 <<  endl\
+							<< "\t\t\tchildren [Bond" << (*l).second.n << " {}]}" << endl;
+				}
+			}
+			n++;
+			(*l).second.l.clear();
+		}
+		LinesMap.clear();
+
+		//end of the world
+		file << "\t]" << endl << "}" << endl;
+
+		g_output_stream_write (output, file.str ().c_str (), file.str ().size (), NULL, &error);
+		if (error) {
+			cerr << "gio error: " << error->message << endl;
+			g_error_free (error);
+			g_object_unref (stream);
+			throw (int) 1;
+		}
+		g_output_stream_close (output, NULL, NULL);
+		g_object_unref (stream);
+	}
+	catch (int n) {
+		// TODO: implement a meaningful error handler.
+	}
+}
+
+void Document::Error (int num) const
+{
+	char const *mess = NULL;
+	GtkWidget* message;
+	char *unescaped = g_uri_unescape_string (m_filename, NULL);
+	switch (num) {
+	case SAVE:
+		mess = _("Could not save file\n%s");
+		break;
+	case LOAD:
+		mess = _("Could not load file\n%s");
+		break;
+	case XML:
+		mess = _("%s: invalid xml file.\nTree is empty?");
+		break;
+	case FORMAT:
+		mess = _("%s: invalid file format.");
+		break;
+	}
+	message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, unescaped);
+	g_free (unescaped);
+	g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
+	gtk_widget_show (message);
+}
+
+bool Document::Load (const std::string &filename)
+{
+	xmlDocPtr xml = NULL;
+	gchar *oldfilename, *oldtitle;
+	// close all dialogs
+	ClearDialogs ();
+	if (m_filename)
+		oldfilename = g_strdup (m_filename);
+	else oldfilename = NULL;
+	oldtitle = g_strdup (m_Title.c_str ());
+	try {
+		if (SetFileName (filename), !m_filename || !m_Label)
+			throw (int) 1;
+		if (!(xml = xmlParseFile (filename.c_str ())))
+			throw (int) 2;
+		if (xml->children == NULL)
+			throw (int) 3;
+		if (strcmp ((char*) xml->children->name, "crystal"))
+			throw (int) 4;
+		if (oldfilename)
+			g_free(oldfilename);
+		g_free (oldtitle);
+		ParseXMLTree (xml->children);
+		xmlFreeDoc (xml);
+		return true;
+	}
+	catch (int num) {
+		switch (num)
+		{
+		case 2:
+		case 3:
+			Error(XML);
+			break;
+		case 4:
+			Error(FORMAT);
+			break;
+		default:
+			Error(LOAD);
+		}
+		if (num > 0) {
+			if (oldfilename)  {
+				SetFileName (oldfilename);
+				g_free (oldfilename);
+			} else {
+				g_free (m_filename);
+				m_filename = NULL;
+			}
+			SetTitle (oldtitle);
+			g_free (oldtitle);
+		}
+		if (num > 1)
+			xmlFreeDoc(xml);
+		return false;
+	}
+}
+
+static int cb_xml_to_vfs (GOutputStream *output, const char* buf, int nb)
+{
+	GError *error = NULL;
+	int n = g_output_stream_write (output, buf, nb, NULL, &error);
+	if (error) {
+		g_message ("GIO error: %s", error->message);
+		g_error_free (error);
+	}
+	return n;
+}
+
+void Document::Save() const
+{
+	if (!m_filename)
+		return;
+	xmlDocPtr xml = NULL;
+
+	try {
+		xml = BuildXMLTree();
+
+		if (!g_date_valid (&m_CreationDate))
+			g_date_set_time_t (&const_cast <Document *> (this)->m_CreationDate, time (NULL));
+		g_date_set_time_t (&const_cast <Document *> (this)->m_RevisionDate, time (NULL));
+		gcu::WriteDate (xml->children, "creation", &m_CreationDate);
+		gcu::WriteDate (xml->children, "revision", &m_RevisionDate);
+		xmlNodePtr node;
+
+		if (m_Title.length () > 0) {
+			node = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("title"), reinterpret_cast <xmlChar const *> (m_Title.c_str ()));
+			if (node)
+				xmlAddChild (xml->children, node);
+			else
+				throw (int) 0;
+		}
+		if ((m_Author && *m_Author) || (m_Mail && *m_Mail)) {
+			node = xmlNewDocNode (xml, NULL, (xmlChar*) "author", NULL);
+			if (node) {
+				if (m_Author && *m_Author)
+					xmlNewProp (node, (xmlChar*) "name", (xmlChar*) m_Author);
+				if (m_Mail && *m_Mail)
+					xmlNewProp (node, (xmlChar*) "e-mail", (xmlChar*) m_Mail);
+				xmlAddChild (xml->children, node);
+			}
+			else
+				throw (int) 0;
+		}
+		if (m_Comment && *m_Comment) {
+			node = xmlNewDocNode (xml, NULL, (xmlChar*) "comment", (xmlChar*) m_Comment);
+			if (node)
+				xmlAddChild (xml->children, node);
+			else
+				throw (int) 0;
+		}
+		xmlIndentTreeOutput = true;
+		xmlKeepBlanksDefault (0);
+
+		xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
+		GFile *file = g_file_new_for_uri (m_filename);
+		GError *error = NULL;
+			if (g_file_query_exists (file, NULL)) {
+				// FIXME: for now, delete it, but we might make a backup?
+				g_file_delete (file, NULL, &error);
+				if (error) {
+					g_message ("GIO error: %s", error->message);
+					g_error_free (error);
+					g_object_unref (file);
+					throw (int) 1;
+				}
+			}
+		GOutputStream *output = G_OUTPUT_STREAM (g_file_create (file, G_FILE_CREATE_NONE, NULL, &error));
+		if (error) {
+			g_message ("GIO error: %s", error->message);
+			g_error_free (error);
+			g_object_unref (file);
+			throw (int) 1;
+		}
+		buf->context = output;
+		buf->closecallback = NULL;
+		buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
+		int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
+		g_output_stream_close (output, NULL, NULL);
+		g_object_unref (file);
+		if (n < 0)
+			throw 1;
+
+		xmlFreeDoc (xml);
+		const_cast <Document *> (this)->SetDirty (false);
+		const_cast <Document *> (this)->m_ReadOnly = false;	// if saving succeded, the file is not read only...
+	}
+	catch (int num) {
+		xmlFreeDoc (xml);
+		Error (SAVE);
+	}
+}
+
+void Document::SaveAsImage (const std::string &filename, char const *type, std::map < std::string, std::string>& options)
+{
+	m_pActiveView->SaveAsImage (filename, type, options, GetApp ()->GetImageWidth (), GetApp ()->GetImageHeight (), !m_App->GetTransparentBackground ());
+}
+
+
 }	//	namespace gcr
