@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Crystal
- * sizedlg.h 
+ * sizedlg.h
  *
- * Copyright (C) 2002-2011 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2012 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,23 +27,40 @@
 
 #include <gcugtk/dialog.h>
 
+/*!\file*/
 namespace gcr {
 
 class Document;
 class Application;
 
+/*!\class SizeDlg gcr/sizedlg.h
+\brief GCrystal model size dialog class.
+
+This class wraps the dialog used to define the size of the view expressed in
+cell coordinates.
+*/
 class SizeDlg: public gcugtk::Dialog
 {
+friend class SizeDlgPrivate;
 public:
+/*!
+@param App the application running the dialog.
+@param pDoc the document.
+
+Creates the dialog.
+*/
 	SizeDlg (Application *App, Document* pDoc);
+/*!
+The destructor.
+*/
 	virtual ~SizeDlg ();
-	
-	virtual bool Apply ();
 
 private:
 	char m_buf[64];
 	Document *m_pDoc;
 	GtkEntry *MaxX, *MinX, *MaxY, *MinY, *MaxZ, *MinZ;
+	unsigned long m_MinXFocusOutSignalID, m_MaxXFocusOutSignalID, m_MinYFocusOutSignalID,
+				   m_MaxYFocusOutSignalID, m_MinZFocusOutSignalID, m_MaxZFocusOutSignalID;
 };
 
 }	//	namespace gcr
