@@ -4,11 +4,11 @@
  * Gnome Crystal library
  * view-settings.h
  *
- * Copyright (C) 2001-2011 Jean Brfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2012 Jean Brfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,15 +27,29 @@
 
 #include <gcugtk/dialog.h>
 
+/*!\file*/
 namespace gcr {
 
 class View;
 
+/*!\class ViewSettingsDlg gcr/view-settings.h
+\brief View settings dialog class.
+
+This class wraps the dialog used to define the view settings.
+*/
 class ViewSettingsDlg: public gcugtk::Dialog
 {
 friend class ViewSettingsDlgPrivate;
 public:
+/*!
+@param pView the view.
+
+Creates the dialog.
+*/
 	ViewSettingsDlg (View* pView);
+/*!
+The destructor.
+*/
 	virtual ~ViewSettingsDlg ();
 
 private:
