@@ -6,7 +6,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -132,7 +132,7 @@ static void on_about (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED Window* Win
 	char const *artists[] = {"Nestor Diaz", NULL};
 	char license[] = "This program is free software; you can redistribute it and/or\n"
 		"modify it under the terms of the GNU General Public License as\n"
- 		"published by the Free Software Foundation; either version 2 of the\n"
+ 		"published by the Free Software Foundation; either version 3 of the\n"
 		"License, or (at your option) any later version.\n\n"
 		"This program is distributed in the hope that it will be useful,\n"
 		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
@@ -205,22 +205,22 @@ static void on_help (G_GNUC_UNUSED GtkWidget *widget, Window* Win)
 
 static void on_web (G_GNUC_UNUSED GtkWidget *widget, Window* Win)
 {
-	static_cast < gcr::Application * > (Win->GetApplication ())->OnWeb ();
+	static_cast < gcr::Application * > (Win->GetApplication ())->OnWeb (Win->GetScreen ());
 }
 
 static void on_mail (G_GNUC_UNUSED GtkWidget *widget, Window* Win)
 {
-	static_cast < gcr::Application * > (Win->GetApplication ())->OnMail ();
+	static_cast < gcr::Application * > (Win->GetApplication ())->OnMail (Win->GetScreen ());
 }
 
 static void on_live_assistance (G_GNUC_UNUSED GtkWidget *widget, Window *Win)
 {
-	Win->GetApplication ()->OnLiveAssistance ();
+	Win->GetApplication ()->OnLiveAssistance (Win->GetScreen ());
 }
 
 static void on_bug (G_GNUC_UNUSED GtkWidget *widget, Window* Win)
 {
-	Win->GetApplication ()->OnBug ();
+	Win->GetApplication ()->OnBug (Win->GetScreen ());
 }
 
 static bool on_focus_in (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED GdkEventFocus *event, Window* Win)
