@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -132,41 +132,6 @@ if they use the FileChooser.
 */
 	void SetCurDir (std::string const &dir);
 
-/*!
-@param uri the uri to display.
-
-Displays uri in the default web browser if any.
-*/
-	void ShowURI (std::string& uri);
-
-/*!
-@param uri the uri to the package bugs database. Default value is PACKAGE_BUGREPORT.
-
-Opens the bugs web page in the default browser if any.
-*/
-	void OnBug (char const *uri = PACKAGE_BUGREPORT)
-		{std::string s (uri); ShowURI (s);}
-
-/*!
-@param uri the uri to the main web page of the program. Default value is
-"http://gchemutils.nongnu.org/".
-*/
-	void OnWeb (char const *uri = "http://gchemutils.nongnu.org/")
-		{std::string s (uri); ShowURI (s);}
-
-/*!
-@param MailAddress the mail adress to which a message will be sent. Defaults to the
-		Gnome Chemistry Utils main list.
-
-Creates a new empty message using the default mail if any.
-*/
-	void OnMail (char const *MailAddress = "mailto:gchemutils-main@nongnu.org");
-
-/*!
-Attempts to open the \#gchemutils channel at irc.gimp.net.
-*/
-	void OnLiveAssistance ();
-
 /*!
 @return a std::map of the supported pixbuf formats. Keys are the mime type names.
 */
@@ -389,7 +354,7 @@ adds a callback for modifying the contextual menu of objects of type Id.
 
 @return the string defined by SetCreationLabel.
 */
-	            const std::string& GetCreationLabel (const std::string& TypeName);
+	const std::string& GetCreationLabel (const std::string& TypeName);
 
 	TypeDesc const *GetTypeDescription (TypeId Id);
 
@@ -438,9 +403,11 @@ This method converts CML to a target.
 */
 	void ConvertFromCML (const char *cml, GsfOutput *output, const char *mime_type, const char *options = NULL);
 
-	            /*!
+/*!
+@param classname the name of a class such as "Molecule".
+@return the list of the databases available for the given classname.
 */
-	            std::list < Database > const &GetDatabases (char const *classname) {return m_Databases[classname];}
+	std::list < Database > const &GetDatabases (char const *classname) {return m_Databases[classname];}
 
 protected:
 /*!
