@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -95,7 +95,7 @@ When pasting, objects added to the document might have the same Id as objects al
 maintains a table to update links using Ids as identifiers. GetTranslatedId returns
 the translated id corresponding to the parameter id.
 */
-	std::string& GetTranslatedId (const char* id) {return m_TranslationTable[id];}
+	std::string& GetTranslatedId (const char* id);
 
 /*!
 @param Id the entry to remove
