@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -35,6 +35,7 @@
 #include <gsf/gsf-output-memory.h>
 #include <glib/gi18n-lib.h>
 #include <stack>
+#include <sstream>
 
 using namespace std;
 
@@ -354,13 +355,25 @@ Molecule *Molecule::MoleculeFromFormula (Document *Doc, Formula const &formula,
 	return mol;
 }
 
-Atom const *Molecule::GetFirstAtom (std::list<Atom*>::const_iterator &i) const
+Atom *Molecule::GetFirstAtom (std::list < Atom * >::iterator &i)
 {
 	i = m_Atoms.begin ();
 	return (i == m_Atoms.end ())? NULL: *i;
 }
 
-Atom const *Molecule::GetNextAtom (std::list<Atom*>::const_iterator &i) const
+Atom const *Molecule::GetFirstAtom (std::list < Atom * >::const_iterator &i) const
+{
+	i = m_Atoms.begin ();
+	return (i == m_Atoms.end ())? NULL: *i;
+}
+
+Atom  *Molecule::GetNextAtom (std::list < Atom* >::iterator &i)
+{
+	i++;
+	return (i == m_Atoms.end ())? NULL: *i;
+}
+
+Atom const *Molecule::GetNextAtom (std::list < Atom * >::const_iterator &i) const
 {
 	i++;
 	return (i == m_Atoms.end ())? NULL: *i;
@@ -478,4 +491,31 @@ void Molecule::ResetIndentifiers ()
 	m_SMILES.clear ();
 }
 
+std::string Molecule::GetRawFormula () const
+{
+	std::ostringstream ofs;
+
+	map<string, int> elts;
+	list<Atom*>::const_iterator ia, enda = m_Atoms.end ();
+	for (ia = m_Atoms.begin(); ia != enda; ia++) {
+		if ((*ia)->GetZ () == 0)
+			continue;
+		elts[(*ia)->GetSymbol ()]++;
+	}
+	if (elts["C"] > 0) {
+		ofs << "C" << elts["C"];
+		elts.erase ("C");
+	}
+	if (elts["H"] > 0) {
+		ofs << "H" << elts["H"];
+		elts.erase ("H");
+	}
+	map<string, int>::iterator is, isend = elts.end ();
+	for (is = elts.begin (); is != isend; is++)
+		ofs << (*is).first << (*is).second;
+
+	return ofs.str ();
+}
+
+
 }	//namespace gcu
