@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -30,11 +30,11 @@
 namespace gcugtk
 {
 
-Chem3dDoc::Chem3dDoc (): gcu::Chem3dDoc ()
+Chem3dDoc::Chem3dDoc (): gcu::Chem3dDoc (Application::GetDefaultApplication (), CreateView ())
 {
 }
 
-Chem3dDoc::Chem3dDoc (Application *App, GLView *View): gcu::Chem3dDoc (App, View)
+Chem3dDoc::Chem3dDoc (Application *App, GLView *View): gcu::Chem3dDoc (App, View? View: CreateView ())
 {
 }
 
