@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * CDXML files loader plugin
- * cdx.cc 
+ * cdx.cc
  *
- * Copyright (C) 2007-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -24,7 +24,6 @@
 
 #include "config.h"
 #include <gcu/application.h>
-#include <gcu/atom.h>
 #include <gcu/bond.h>
 #include <gcu/document.h>
 #include <gcu/element.h>
@@ -33,6 +32,8 @@
 #include <gcu/molecule.h>
 #include <gcu/objprops.h>
 #include <gcu/residue.h>
+#include <gcp/atom.h>
+#include <gcp/fragment.h>
 
 #include <goffice/app/module-plugin-defs.h>
 #include <gsf/gsf-output-memory.h>
@@ -125,7 +126,7 @@ public:
 	virtual ~CDXLoader ();
 
 	ContentType Read (Document *doc, GsfInput *in, char const *mime_type, GOIOContext *io);
-	bool Write (Object *obj, GsfOutput *out, char const *mime_type, GOIOContext *io, ContentType type);
+	bool Write (Object const *obj, GsfOutput *out, char const *mime_type, GOIOContext *io, ContentType type);
 
 private:
 	bool ReadGenericObject (GsfInput *in);
@@ -140,14 +141,14 @@ private:
 	guint16 ReadSize (GsfInput *in);
 	bool ReadDate (GsfInput *in);
 
-	bool WriteObject (GsfOutput *out, Object *object, GOIOContext *io);
+	bool WriteObject (GsfOutput *out, Object const *object, GOIOContext *io);
 	static void AddInt16Property (GsfOutput *out, gint16 prop, gint16 value);
 	static void AddInt32Property (GsfOutput *out, gint16 prop, gint32 value);
 	static void WriteSimpleStringProperty (GsfOutput *out, gint16 id, gint16 length, char const *data);
-	static bool WriteAtom (CDXLoader *loader, GsfOutput *out, Object *obj, GOIOContext *s);
-	static bool WriteBond (CDXLoader *loader, GsfOutput *out, Object *obj, GOIOContext *s);
-	static bool WriteMolecule (CDXLoader *loader, GsfOutput *out, Object *obj, GOIOContext *s);
-	void WriteId (Object *obj, GsfOutput *out);
+	static bool WriteAtom (CDXLoader *loader, GsfOutput *out, Object const *obj, GOIOContext *s);
+	static bool WriteBond (CDXLoader *loader, GsfOutput *out, Object const *obj, GOIOContext *s);
+	static bool WriteMolecule (CDXLoader *loader, GsfOutput *out, Object const *obj, GOIOContext *s);
+	void WriteId (Object const *obj, GsfOutput *out);
 
 private:
 	char *buf;
@@ -156,7 +157,7 @@ private:
 	vector <string> colors;
 	guint8 m_TextAlign, m_TextJustify;
 
-	map <string, bool (*) (CDXLoader *, GsfOutput *, Object *, GOIOContext *)> m_WriteCallbacks;
+	map <string, bool (*) (CDXLoader *, GsfOutput *, Object const *, GOIOContext *)> m_WriteCallbacks;
 	map<unsigned, GOColor> m_Colors;
 	map <string, gint32> m_SavedIds;
 	gint32 m_MaxId;
@@ -229,7 +230,7 @@ ContentType CDXLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 				doc->SetProperty (GCU_PROP_DOC_CREATION_TIME, buf);
 				break;
 			}
-			case kCDXProp_ModificationDate:{ 
+			case kCDXProp_ModificationDate:{
 				if (size != 14 || !ReadDate (in)) {
 					result = ContentTypeUnknown;
 					break;
@@ -256,8 +257,9 @@ ContentType CDXLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 					result = ContentTypeUnknown;
 					break;
 				}
-				snprintf (buf, bufsize, "%u", length);
-				doc->SetProperty (GCU_PROP_THEME_BOND_LENGTH, buf);
+				ostringstream str;
+				str << length;
+				doc->SetProperty (GCU_PROP_THEME_BOND_LENGTH, str.str ().c_str ());
 				break;
 			}
 			case kCDXProp_FontTable: {
@@ -288,8 +290,9 @@ ContentType CDXLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 				for (unsigned i = 0; i < nb; i++) {
 				if (!(READINT16 (in,red)) || !(READINT16 (in,green)) || !(READINT16 (in,blue)))
 					return ContentTypeUnknown;
-					snprintf (buf, bufsize, "red=\"%g\" green=\"%g\" blue=\"%g\"", (double) red / 0xffff, (double) green / 0xffff, (double) blue / 0xffff);
-					colors.push_back (buf);
+					ostringstream str;
+					str << "red=\"" << (double) red / 0xffff << "\" green=\"" << (double) green / 0xffff << "\" blue=\"" << (double) blue / 0xffff << "\"";
+					colors.push_back (str.str ());
 				}
 				break;
 			}
@@ -319,8 +322,8 @@ ContentType CDXLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char co
 /******************************************************************************
  *	Write callbacks															  *
  ******************************************************************************/
- 
-bool CDXLoader::WriteAtom (CDXLoader *loader, GsfOutput *out, Object *obj, G_GNUC_UNUSED GOIOContext *s)
+
+bool CDXLoader::WriteAtom (CDXLoader *loader, GsfOutput *out, Object const *obj, G_GNUC_UNUSED GOIOContext *s)
 {
 	gint16 n = kCDXObj_Node;
 	double x, y;
@@ -329,7 +332,8 @@ bool CDXLoader::WriteAtom (CDXLoader *loader, GsfOutput *out, Object *obj, G_GNU
 	loader->WriteId (obj, out);
 	string prop = obj->GetProperty (GCU_PROP_POS2D);
 	if (prop.length ()) {
-		sscanf (prop.c_str (), "%lg %lg", &x, &y);
+		istringstream str (prop);
+		str >> x >> y;
 		x_ = x;
 		y_ = y;
 		n = kCDXProp_2DPosition;
@@ -339,7 +343,7 @@ bool CDXLoader::WriteAtom (CDXLoader *loader, GsfOutput *out, Object *obj, G_GNU
 		WRITEINT32 (out, y_);
 		WRITEINT32 (out, x_);
 	}
-	AddInt16Property (out, kCDXProp_ZOrder, loader->m_Z++);	
+	AddInt16Property (out, kCDXProp_ZOrder, loader->m_Z++);
 	prop = obj->GetProperty (GCU_PROP_ATOM_Z);
 	if (prop != "6") {
 		n = kCDXProp_Node_Element;
@@ -352,40 +356,40 @@ bool CDXLoader::WriteAtom (CDXLoader *loader, GsfOutput *out, Object *obj, G_GNU
 	return true;
 }
 
-bool CDXLoader::WriteBond (CDXLoader *loader, GsfOutput *out, Object *obj, G_GNUC_UNUSED GOIOContext *s)
+bool CDXLoader::WriteBond (CDXLoader *loader, GsfOutput *out, Object const *obj, G_GNUC_UNUSED GOIOContext *s)
 {
 	gint16 n = kCDXObj_Bond;
 	WRITEINT16 (out, n);
 	loader->WriteId (obj, out);
-	AddInt16Property (out, kCDXProp_ZOrder, loader->m_Z++);	
+	AddInt16Property (out, kCDXProp_ZOrder, loader->m_Z++);
 	string prop = obj->GetProperty (GCU_PROP_BOND_BEGIN);
 	AddInt32Property (out, kCDXProp_Bond_Begin, loader->m_SavedIds[prop]);
 	prop = obj->GetProperty (GCU_PROP_BOND_END);
 	AddInt32Property (out, kCDXProp_Bond_End, loader->m_SavedIds[prop]);
 	prop = obj->GetProperty (GCU_PROP_BOND_ORDER);
 	if (prop == "3")
-		AddInt16Property (out, kCDXProp_Bond_Order, 4);	
+		AddInt16Property (out, kCDXProp_Bond_Order, 4);
 	else if (prop == "2")
-		AddInt16Property (out, kCDXProp_Bond_Order, 2);	
+		AddInt16Property (out, kCDXProp_Bond_Order, 2);
 	prop = obj->GetProperty (GCU_PROP_BOND_TYPE);
 	if (prop == "wedge")
-		AddInt16Property (out, kCDXProp_Bond_Display, 6);	
+		AddInt16Property (out, kCDXProp_Bond_Display, 6);
 	else if (prop == "hash")
-		AddInt16Property (out, kCDXProp_Bond_Display, 3);	
+		AddInt16Property (out, kCDXProp_Bond_Display, 3);
 	else if (prop == "squiggle")
-		AddInt16Property (out, kCDXProp_Bond_Display, 8);	
+		AddInt16Property (out, kCDXProp_Bond_Display, 8);
 	gsf_output_write (out, 2, reinterpret_cast <guint8 const *> ("\x00\x00")); // end of bond
 	return true;
 }
 
-bool CDXLoader::WriteMolecule (CDXLoader *loader, GsfOutput *out, Object *obj, GOIOContext *s)
+bool CDXLoader::WriteMolecule (CDXLoader *loader, GsfOutput *out, Object const *obj, GOIOContext *s)
 {
 	gint16 n = kCDXObj_Fragment;
 	WRITEINT16 (out, n);
 	loader->WriteId (obj, out);
 	// save atoms
-	std::map <std::string, Object *>::iterator i;
-	Object *child = obj->GetFirstChild (i);
+	std::map <std::string, Object *>::const_iterator i;
+	Object const *child = obj->GetFirstChild (i);
 	while (child) {
 		if (child->GetType () == AtomType && !loader->WriteObject (out, child, s))
 			return false;
@@ -409,15 +413,15 @@ bool CDXLoader::WriteMolecule (CDXLoader *loader, GsfOutput *out, Object *obj, G
 	return true;
 }
 
-bool CDXLoader::WriteObject (GsfOutput *out, Object *object, GOIOContext *io)
+bool CDXLoader::WriteObject (GsfOutput *out, Object const *object, GOIOContext *io)
 {
 	string name = Object::GetTypeName (object->GetType ());
-	map <string, bool (*) (CDXLoader *, GsfOutput *, Object *, GOIOContext *)>::iterator i = m_WriteCallbacks.find (name);
+	map <string, bool (*) (CDXLoader *, GsfOutput *, Object const *, GOIOContext *)>::iterator i = m_WriteCallbacks.find (name);
 	if (i != m_WriteCallbacks.end ())
 		return (*i).second (this, out, object, io);
 	// if we don't save the object iself, try tosave its children
-	std::map <std::string, Object *>::iterator j;
-	Object *child = object->GetFirstChild (j);
+	std::map <std::string, Object *>::const_iterator j;
+	Object const *child = object->GetFirstChild (j);
 	while (child) {
 		if (!WriteObject (out, child, io))
 			return false;
@@ -436,7 +440,7 @@ void CDXLoader::WriteSimpleStringProperty (GsfOutput *out, gint16 id, gint16 len
 	gsf_output_write (out, length, reinterpret_cast <guint8 const *> (data));
 }
 
-void CDXLoader::WriteId (Object *obj, GsfOutput *out)
+void CDXLoader::WriteId (Object const *obj, GsfOutput *out)
 {
 	m_SavedIds[obj->GetId ()] = m_MaxId;
 	gint32 n = m_MaxId++;
@@ -457,11 +461,12 @@ void CDXLoader::AddInt32Property (GsfOutput *out, gint16 prop, gint32 value)
 	WRITEINT32 (out, value);
 }
 
-bool CDXLoader::Write  (Object *obj, GsfOutput *out, G_GNUC_UNUSED G_GNUC_UNUSED char const *mime_type, GOIOContext *io, G_GNUC_UNUSED ContentType type)
+bool CDXLoader::Write  (Object const *obj, GsfOutput *out, G_GNUC_UNUSED G_GNUC_UNUSED char const *mime_type, GOIOContext *io, G_GNUC_UNUSED ContentType type)
 {
-	Document *doc = dynamic_cast <Document *> (obj);
+	Document const *doc = dynamic_cast <Document const *> (obj);
 	gint16 n;
 	gint32 l;
+	// FIXME: should be able to export a molecule or any object actually
 	if (!doc || !out)
 		return false;
 
@@ -488,7 +493,7 @@ bool CDXLoader::Write  (Object *obj, GsfOutput *out, G_GNUC_UNUSED G_GNUC_UNUSED
 	// determine the bond length and scale the document appropriately
 	string prop = doc->GetProperty (GCU_PROP_THEME_BOND_LENGTH);
 	double scale = strtod (prop.c_str (), NULL);
-	doc->SetScale (scale / 1966080.);
+	const_cast <Document *> (doc)->SetScale (scale / 1966080.);
 	n = kCDXProp_BondLength;
 	WRITEINT16 (out, n);
 	gsf_output_write (out, 2, reinterpret_cast <guint8 const *> ("\x04\x00"));
@@ -504,8 +509,8 @@ bool CDXLoader::Write  (Object *obj, GsfOutput *out, G_GNUC_UNUSED G_GNUC_UNUSED
 	WRITEINT16 (buf, n);
 	l = 0;
 	WRITEINT32 (buf, l); // id = 0 for the page
-	std::map <std::string, Object *>::iterator i;
-	Object *child = doc->GetFirstChild (i);
+	std::map <std::string, Object *>::const_iterator i;
+	Object const *child = doc->GetFirstChild (i);
 	while (child) {
 		if (!WriteObject (buf, child, io)) {
 			g_object_unref (buf);
@@ -640,8 +645,9 @@ bool CDXLoader::ReadMolecule (GsfInput *in, Object *parent)
 	guint32 Id;
 	if (!(READINT32 (in,Id)))
 		return false;
-	snprintf (buf, bufsize, "m%d", Id);
-	mol->SetId (buf);
+	ostringstream str;
+	str << "m" << Id;
+	mol->SetId (str.str ().c_str ());
 	if (!(READINT16 (in,code)))
 		return false;
 	while (code) {
@@ -670,6 +676,7 @@ bool CDXLoader::ReadMolecule (GsfInput *in, Object *parent)
 			return false;
 	}
 		static_cast <Molecule*> (mol)->UpdateCycles ();
+	parent->GetDocument ()->ObjectLoaded (mol);
 	return true;
 }
 
@@ -684,8 +691,9 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 	int Z = 6;
 	if (!(READINT32 (in,Id)))
 		return false;
-	snprintf (buf, bufsize, "a%d", Id);
-	Atom->SetId (buf);
+	ostringstream str;
+	str << "a" << Id;
+	Atom->SetId (str.str ().c_str ());
 	if (!(READINT16 (in,code)))
 		return false;
 	while (code) {
@@ -738,8 +746,9 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 								delete Atom;
 								Atom = parent->GetApplication ()->CreateObject ("fragment", parent);
 								Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-								snprintf (buf, bufsize, "a%d", Id);
-								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+								ostringstream str;
+								str << "a" << Id;
+								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, str.str ().c_str ());
 								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
 								Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
 							}
@@ -767,17 +776,18 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 								delete Atom;
 								Atom = parent->GetApplication ()->CreateObject ("fragment", parent);
 								Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-								snprintf (buf, bufsize, "a%d", Id);
-								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+								ostringstream str;
+								str << "a" << Id;
+								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, str.str ().c_str ());
 								Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
 								Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
 							} else {
-								// FIXME: should the document care with the residues?
 								g_warning (_("Unsupported feature, please report!"));
+								// FIXME: should the document care with the residues?
 							}
 						} else {
-							// FIXME: Unkown residue: add it to the database? or just to the document?
 							g_warning (_("Unsupported feature, please report!"));
+							// FIXME: Unkown residue: add it to the database? or just to the document?
 						}
 						break;
 					}
@@ -828,7 +838,7 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 										switch (valence) {
 										case 2: {
 											/* remove the first atom and replace it by a pseudo-atom, then add the residue
-											this helps with things begining with an oxygen or a sulfur, but might be 
+											this helps with things begining with an oxygen or a sulfur, but might be
 											not enough n other cases */
 											double x, y;
 											a2->GetCoords (&x, &y);
@@ -879,8 +889,9 @@ fragment_success:
 							delete Atom;
 							Atom = parent->GetApplication ()->CreateObject ("fragment", parent);
 							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-							snprintf (buf, bufsize, "a%d", Id);
-							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+							ostringstream str;
+							str << "a" << Id;
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, str.str ().c_str ());
 							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
 							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
 						}
@@ -896,13 +907,15 @@ fragment_success:
 							delete Atom;
 							Atom = parent->GetApplication ()->CreateObject ("fragment", parent);
 							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
-							snprintf (buf, bufsize, "a%d", Id);
-							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+							ostringstream str;
+							str << "a" << Id;
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, str.str ().c_str ());
 							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
 							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+							parent->GetDocument ()->ObjectLoaded (static_cast <gcp::Fragment *> (Atom)->GetAtom ());
 						} else {
-							// TODO: import it in the document
 							g_warning (_("Unsupported feature, please report!"));
+							// TODO: import it in the document
 						}
 						break;
 					}
@@ -925,24 +938,29 @@ fragment_success:
 				gint32 x, y;
 				if (size != 8 || !(READINT32 (in,y)) || !(READINT32 (in,x)))
 					goto bad_exit;
-				snprintf (buf, bufsize, "%d %d", x, y);
-				Atom->SetProperty (GCU_PROP_POS2D, buf);
+				ostringstream str;
+				str <<  x << " " << y;
+				Atom->SetProperty (GCU_PROP_POS2D, str.str ().c_str ());
 				break;
 			}
-			case kCDXProp_Node_Element:
+			case kCDXProp_Node_Element: {
 				if (size != 2 || !(READINT16 (in,size)))
 					goto bad_exit;
 				Z = size;
-				snprintf (buf, bufsize, "%u", size);
-				Atom->SetProperty (GCU_PROP_ATOM_Z, buf);
+				ostringstream str;
+				str <<  size;
+				Atom->SetProperty (GCU_PROP_ATOM_Z, str.str ().c_str ());
 				break;
-			case kCDXProp_Atom_Charge:
+			}
+			case kCDXProp_Atom_Charge: {
 				gint8 charge;
 				if (size!= 1 || !gsf_input_read (in, 1, (guint8*) &charge))
 					goto bad_exit;
-				snprintf (buf, bufsize, "%d", charge);
-				Atom->SetProperty (GCU_PROP_ATOM_CHARGE, buf);
+				ostringstream str;
+				str <<  charge;
+				Atom->SetProperty (GCU_PROP_ATOM_CHARGE, str.str ().c_str ());
 				break;
+			}
 			case kCDXProp_Node_Type:
 				if (size != 2 || !(READINT16 (in,type)))
 					goto bad_exit;
@@ -954,8 +972,9 @@ fragment_success:
 						mol->Remove (Atom);
 					delete Atom;
 					Atom = parent->GetApplication ()->CreateObject ("pseudo-atom", parent);
-					snprintf (buf, bufsize, "a%d", Id);
-					Atom->SetId (buf);
+					ostringstream str;
+					str << "a" << Id;
+					Atom->SetId (str.str ().c_str ());
 					Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
 				}
 				break;
@@ -969,6 +988,7 @@ fragment_success:
 	}
 	if (Doc)
 		delete Doc;
+	parent->GetDocument ()->ObjectLoaded (Atom);
 	return true;
 bad_exit:
 	if (Doc)
@@ -983,8 +1003,9 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 	guint32 Id;
 	if (!(READINT32 (in,Id)))
 		return false;
-	snprintf (buf, bufsize, "b%d", Id);
-	Bond->SetId (buf);
+	ostringstream str;
+	str << "b" << Id;
+	Bond->SetId (str.str ().c_str ());
 	Bond->SetProperty (GCU_PROP_BOND_ORDER, "1");
 	if (!(READINT16 (in,code)))
 		return false;
@@ -1000,15 +1021,17 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 			case kCDXProp_Bond_Begin: {
 				if (size != 4 || !(READINT32 (in,Id)))
 					return false;
-				snprintf (buf, bufsize, "%u", Id);
-				Bond->SetProperty (GCU_PROP_BOND_BEGIN, buf);
+				ostringstream str;
+				str << Id;
+				Bond->SetProperty (GCU_PROP_BOND_BEGIN, str.str ().c_str ());
 				break;
 			}
 			case kCDXProp_Bond_End: {
 				if (size != 4 || !(READINT32 (in,Id)))
 					return false;
-				snprintf (buf, bufsize, "%u", Id);
-				Bond->SetProperty (GCU_PROP_BOND_END, buf);
+				ostringstream str;
+				str << Id;
+				Bond->SetProperty (GCU_PROP_BOND_END, str.str ().c_str ());
 				break;
 			}
 			case kCDXProp_Bond_Order:
@@ -1062,6 +1085,7 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 		if (!(READINT16 (in,code)))
 			return false;
 	}
+	parent->GetDocument ()->ObjectLoaded (Bond);
 	return true;
 }
 
@@ -1081,8 +1105,9 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 	guint8 TextAlign = 0xfe, TextJustify = 0xfe;
 	if (!(READINT32 (in,Id)))
 		return false;
-	snprintf (buf, bufsize, "t%d", Id);
-	Text->SetId (buf);
+	ostringstream str;
+	str << "t" << Id;
+	Text->SetId (str.str ().c_str ());
 	if (!(READINT16 (in,code)))
 		return false;
 	while (code) {
@@ -1102,8 +1127,9 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 					return false;
 				if (!(READINT32 (in,x)))
 					return false;
-				snprintf (buf, bufsize, "%d %d", x, y);
-				Text->SetProperty (GCU_PROP_POS2D, buf);
+				ostringstream str;
+				str <<  x << " " << y;
+				Text->SetProperty (GCU_PROP_POS2D, str.str ().c_str ());
 				break;
 			}
 			case kCDXProp_Text: {
@@ -1150,8 +1176,8 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 								// FIXME: fix this kludgy code
 								int cur = 0;
 								while (cur < attrs0.index) {
-									while (cur < attrs0.index && (buf[cur] < '0' || buf[cur] > '9')){printf("cur=%d c=%c\n",cur,buf[cur]);
-										str << buf[cur++];}
+									while (cur < attrs0.index && (buf[cur] < '0' || buf[cur] > '9'))
+										str << buf[cur++];
 									if (cur < attrs0.index) {
 										if (attrs0.face & 4)
 											str << "</u>";
@@ -1162,8 +1188,8 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 										str << "</fore></font><font name=\"" << m_Fonts[attrs.font].name << " " << (double) attrs.size / 30. << "\">";
 										str << "<fore " << colors[attrs.color] << ">";
 										str << "<sub height=\"" << (double) attrs.size / 60. << "\">";
-										while (buf[cur] >= '0' && buf[cur] <= '9'){printf("cur=%d c=%c\n",cur,buf[cur]);
-											str << buf[cur++];}
+										while (buf[cur] >= '0' && buf[cur] <= '9')
+											str << buf[cur++];
 										str << "</sub></fore></font><font name=\"" << m_Fonts[attrs.font].name << " " << (double) attrs.size / 20. << "\">";
 										str << "<fore " << colors[attrs.color] << ">";
 										if (attrs0.face & 1)
@@ -1292,13 +1318,13 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 	if (TextAlign == 0xfe)
 		TextAlign = m_TextAlign;
 	switch (TextAlign) {
-	case 0xff: 
+	case 0xff:
 		Text->SetProperty (GCU_PROP_TEXT_ALIGNMENT, "right");
 		break;
-	case 0: 
+	case 0:
 		Text->SetProperty (GCU_PROP_TEXT_ALIGNMENT, "left");
 		break;
-	case 1: 
+	case 1:
 		Text->SetProperty (GCU_PROP_TEXT_ALIGNMENT, "center");
 		break;
 	default:
@@ -1308,22 +1334,23 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 	if (TextJustify == 0xfe)
 		TextJustify = m_TextJustify;
 	switch (TextJustify) {
-	case 0xff: 
+	case 0xff:
 		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "right");
 		break;
-	case 0: 
+	case 0:
 		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "left");
 		break;
-	case 1: 
+	case 1:
 		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "center");
 		break;
-	case 2: 
+	case 2:
 		Text->SetProperty (GCU_PROP_TEXT_JUSTIFICATION, "justify");
 		break;
 	default:
 	// Other cases are not currently supported
 		break;
 	}
+	parent->GetDocument ()->ObjectLoaded (Text);
 	return true;
 }
 
@@ -1363,6 +1390,7 @@ bool CDXLoader::ReadGroup (GsfInput *in, Object *parent)
 	}
 	Group->Lock (false);
 	Group->OnLoaded ();
+	parent->GetDocument ()->ObjectLoaded (Group);
 	return true;
 }
 
@@ -1404,32 +1432,35 @@ bool CDXLoader::ReadGraphic  (GsfInput *in, Object *parent)
 	}
 	if (type == 1) {
 		Object *obj = NULL;
+		ostringstream str;
 		switch (arrow_type) {
 		case 1:
 		case 2:
 			obj = parent->GetApplication ()->CreateObject ("reaction-arrow", parent);
-			snprintf (buf, bufsize, "ra%d", Id);
+			str << "ra" << Id;
 			break;
 		case 4:
 			obj = parent->GetApplication ()->CreateObject ("mesomery-arrow", parent);
-			snprintf (buf, bufsize, "ma%d", Id);
+			str << "ma" << Id;
 			break;
 		case 8:
 			obj = parent->GetApplication ()->CreateObject ("reaction-arrow", parent);
-			snprintf (buf, bufsize, "ra%d", Id);
+			str << "ra" << Id;
 			obj->SetProperty (GCU_PROP_REACTION_ARROW_TYPE, "double");
 			break;
 		case 32:
 			obj = parent->GetApplication ()->CreateObject ("retrosynthesis-arrow", parent);
-			snprintf (buf, bufsize, "rsa%d", Id);
+			str << "rsa" << Id;
 			break;
 		default:
 			break;
 		}
 		if (obj) {
-			obj->SetId (buf);
-			snprintf (buf, bufsize, "%d %d %d %d", x0, y0, x1, y1);
-			obj->SetProperty (GCU_PROP_ARROW_COORDS, buf);
+			obj->SetId (str.str ().c_str ());
+			ostringstream str_;
+			str_ << x0 << " " << y0 << " " << x1 << " " << y1;
+			obj->SetProperty (GCU_PROP_ARROW_COORDS, str_.str ().c_str ());
+			parent->GetDocument ()->ObjectLoaded (obj);
 		}
 	}
 	return true;
@@ -1445,7 +1476,7 @@ guint16 CDXLoader::ReadSize  (GsfInput *in)
 			bufsize <<= 1;
 		while ((unsigned) size + 1 > bufsize);
 		delete [] buf;
-		buf = new char [bufsize];	
+		buf = new char [bufsize];
 	}
 	return size;
 }
@@ -1486,8 +1517,9 @@ bool CDXLoader::ReadFragmentText (GsfInput *in, G_GNUC_UNUSED Object *parent)
 					return false;
 				if (!READINT32 (in, x))
 					return false;
-				snprintf (buf, bufsize, "%d %d", x, y);
-//				Text->SetProperty (GCU_PROP_POS2D, buf);
+				ostringstream str;
+				str <<  x << " " << y;
+				Text->SetProperty (GCU_PROP_POS2D, str.str ().c_str ());
 				break;
 			}*/
 			case kCDXProp_Text: {
