@@ -52,6 +52,10 @@ gcpBracketsTool::gcpBracketsTool (gcp::Application* App): gcp::Tool (App, "Brack
 	m_Type = gccv::BracketsTypeNormal;
 	m_Used = gccv::BracketsBoth;
 	m_FontDesc = pango_font_description_new ();
+	m_ActualBounds.x0 = 0.;
+	m_ActualBounds.y0 = 0.;
+	m_ActualBounds.x1 = 0.;
+	m_ActualBounds.y1 = 0.;
 }
 
 gcpBracketsTool::~gcpBracketsTool ()
@@ -99,12 +103,13 @@ void gcpBracketsTool::OnDrag ()
 					y0 *= m_dZoomFactor;
 					if (x0 >= xmin && x0 <= xmax && y0 >= ymin && y0 <= ymax) {
 						m_pData->SetSelected (object);
-						gcp::Atom *atom;
+						gcp::Atom *atom = NULL;
 						switch (object->GetType ()) {
 						case gcu::FragmentType:
 							atom = static_cast <gcp::Fragment *> (object)->GetAtom ();
 						case gcu::AtomType: {
-							atom = static_cast <gcp::Atom *> (object);
+							if (atom == NULL)
+								atom = static_cast <gcp::Atom *> (object);
 							// go through the bonds and select them if both ends are selected
 							std::map<gcu::Atom*, gcu::Bond*>::iterator i;
 							gcu::Bond *bond = atom->GetFirstBond (i);
