@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * GChemPaint selection plugin
- * erasertool.h 
+ * erasertool.h
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,17 +27,21 @@
 
 #include <gcp/tool.h>
 
-class gcp::Application;
-	
+namespace gcp {
+	class Application;
+}
+
 class gcpEraserTool: public gcp::Tool
 {
 public:
 	gcpEraserTool (gcp::Application* App);
 	virtual ~gcpEraserTool ();
 
-	virtual bool OnClicked ();
-	virtual void OnDrag ();
-	virtual void OnRelease ();
+	bool OnClicked ();
+	void OnDrag ();
+	void OnRelease ();
+	void OnMotion ();
+	void OnLeaveNotify ();
 	char const *GetHelpTag () {return "eraser";}
 };
 
