@@ -4,11 +4,11 @@
  * Gnome Chemistry Utils
  * programs/calc/gchemcalc.cc
  *
- * Copyright (C) 2005-2011 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2012 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,6 +25,7 @@
 #include "config.h"
 #include <gcugtk/application.h>
 #include <gcugtk/filechooser.h>
+#include <gcugtk/message.h>
 #include <gcugtk/printable.h>
 #include <gcugtk/print-setup-dlg.h>
 #include <gcugtk/ui-builder.h>
@@ -198,6 +199,8 @@ void GChemCalc::OnSaveAsImage ()
 	gcugtk::FileChooser (this, true, l, NULL, _("Save as image"), GetImageSizeWidget ());
 }
 
+GChemCalc *App;
+
 bool GChemCalc::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, G_GNUC_UNUSED GtkWindow *window, G_GNUC_UNUSED Document *pDoc)
 {
 	if(bSave) {
@@ -208,10 +211,9 @@ bool GChemCalc::FileProcess (const gchar* filename, const gchar* mime_type, bool
 			char *unescaped = g_uri_unescape_string (filename, NULL);
 			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), unescaped);
 			g_free (unescaped);
-			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-			gtk_window_set_icon_name (GTK_WINDOW (Box), "gchemcalc");
-			result = gtk_dialog_run (Box);
-			gtk_widget_destroy (GTK_WIDGET (Box));
+			gcugtk::Message *box = new gcugtk::Message (static_cast < gcugtk::Application * > (App),
+			                                            message, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, App->GetGtkWindow ());
+			box->Run ();
 			g_free (message);
 		}
 		if (result == GTK_RESPONSE_YES) {
@@ -243,8 +245,6 @@ void GChemCalc::OnSize (GChemCalc *calc, GtkAllocation *allocation)
 	calc->m_GraphHeight = allocation->height;
 }
 
-GChemCalc *App;
-
 static void on_quit (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED void *data)
 {
 	gtk_widget_destroy (GTK_WIDGET (App->window));
@@ -255,24 +255,24 @@ static void on_help (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer dat
 	App->OnHelp ();
 }
 
-static void on_web (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
+static void on_web (GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
-	App->OnWeb ();
+	App->OnWeb (gtk_widget_get_screen (widget));
 }
 
-static void on_mail (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
+static void on_mail (GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
-	App->OnMail ();
+	App->OnMail (gtk_widget_get_screen (widget));
 }
 
-static void on_live_assistance (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer dataw)
+static void on_live_assistance (GtkWidget *widget, G_GNUC_UNUSED gpointer dataw)
 {
-	App->OnLiveAssistance ();
+	App->OnLiveAssistance (gtk_widget_get_screen (widget));
 }
 
-static void on_bug (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED gpointer data)
+static void on_bug (GtkWidget *widget, G_GNUC_UNUSED gpointer data)
 {
-	App->OnBug ();
+	App->OnBug (gtk_widget_get_screen (widget));
 }
 
 static void on_about (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED void *data)
@@ -284,7 +284,7 @@ static void on_about (G_GNUC_UNUSED GtkWidget *widget, G_GNUC_UNUSED void *data)
 	const gchar * license =
 		"This program is free software; you can redistribute it and/or\n"
 		"modify it under the terms of the GNU General Public License as\n"
-		"published by the Free Software Foundation; either version 2 of the\n"
+		"published by the Free Software Foundation; either version 3 of the\n"
 		"License, or (at your option) any later version.\n\n"
 		"This program is distributed in the hope that it will be useful,\n"
 		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
@@ -315,7 +315,7 @@ static void clear_values (double *values)
 	delete [] values;
 }
 
-static void cb_entry_active (GtkEntry *entry, gpointer data)
+static void cb_entry_active (GtkEntry *entry, gpointer)
 {
 	GError *error;
 	try {
@@ -456,13 +456,9 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 		int start, length;
 		char const *mess = error.what (start, length);
 		gtk_editable_select_region (GTK_EDITABLE (entry), start, start + length);
-		GtkWidget *w = gtk_message_dialog_new (GTK_WINDOW (data),
-							GTK_DIALOG_DESTROY_WITH_PARENT,
-							GTK_MESSAGE_ERROR,
-							GTK_BUTTONS_OK,
-							mess);
-		g_signal_connect_swapped (G_OBJECT (w), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (w));
-		gtk_widget_show (w);
+		gcugtk::Message *box = new gcugtk::Message (static_cast < gcugtk::Application * > (App),
+		                                            mess, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, App->GetGtkWindow ());
+		box->Show ();
 	}
 }
 
@@ -684,8 +680,9 @@ int main (int argc, char *argv[])
 {
 	GOptionContext *context;
 	GError *error = NULL;
-	textdomain (GETTEXT_PACKAGE);
 	gtk_init (&argc, &argv);
+	gcu_element_load_databases ("isotopes", NULL);
+	Element::LoadBODR ();
 	if (argc > 1 && argv[1][0] == '-') {
 		context = g_option_context_new (_(" [formula]"));
 		g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
@@ -795,8 +792,6 @@ int main (int argc, char *argv[])
 	g_signal_connect (G_OBJECT (w), "activate",
 		 G_CALLBACK (cb_entry_active),
 		 App->window);
-	gcu_element_load_databases ("isotopes", NULL);
-	Element::LoadBODR ();
 	if (argc == 1){
 		gtk_entry_set_text (GTK_ENTRY (w), argv[0]);
 		cb_entry_active (GTK_ENTRY (w), App->window);
