@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Crystal
- * cleavage.h 
+ * cleavage.h
  *
- * Copyright (C) 2001-2004 Jean Brfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2010 Jean Brfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,23 +25,17 @@
 #ifndef GCRYSTAL_CLEAVAGE_H
 #define GCRYSTAL_CLEAVAGE_H
 
-#ifdef GCU_OLD_VER
-#	include <crystalviewer/crystalcleavage.h>
-#else
-#	include <gcu/crystalcleavage.h>
-#endif
+#include <gcr/cleavage.h>
 #include <libxml/parser.h>
 #include <list>
 
-using namespace gcu;
-
-class gcCleavage: public CrystalCleavage
+class gcCleavage: public gcr::Cleavage
 {
 public:
-	gcCleavage();
-	virtual ~gcCleavage();
+	gcCleavage ();
+	virtual ~gcCleavage ();
 
-	bool LoadOld(xmlNodePtr node);
+	bool LoadOld (xmlNodePtr node);
 };
 
 #endif //GCRYSTAL_CLEAVAGE_H
