@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Crystal
- * view.cc 
+ * view.cc
  *
- * Copyright (C) 2000-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2000-2011 Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,33 +25,21 @@
 #include "config.h"
 #include <GL/gl.h>
 #include <GL/glu.h>
-#include <gtk/gtkgl.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
 #include "view.h"
 #include "document.h"
-#include "globals.h"
 #include <cmath>
 #include <cstring>
 
-guint FoV;
-gdouble Psi, Theta, Phi;
-gdouble Red, Green, Blue;
 extern GtkWidget *vbox1;
 
-gcView::gcView (gcDocument *pDoc): CrystalView (pDoc)
+gcView::gcView (gcDocument *pDoc): gcr::View (pDoc)
 {
-	SetAngle (FoV);
-	SetRotation (Psi, Theta, Phi);
-	SetBlue (Blue);
-	SetRed (Red);
-	SetGreen (Green);
-	SetAlpha (1.0);
-	m_Window = NULL;
 }
 
 gcView::gcView (gcView *pView):
-	CrystalView (dynamic_cast <CrystalDoc *> (pView->GetDoc ()))
+	gcr::View (dynamic_cast <gcr::Document *> (pView->GetDoc ()))
 {
 	SetAngle (pView->GetAngle ());
 	SetRotation (pView->GetPsi (), pView->GetTheta (), pView->GetPhi ());
@@ -64,62 +52,27 @@ gcView::gcView (gcView *pView):
 
 gcView::~gcView ()
 {
-	gtk_widget_destroy(GTK_WIDGET(m_pMenu));
-	Dialog *dialog;
-	while (!m_Dialogs.empty ()) {
-		dialog = m_Dialogs.front();
-		m_Dialogs.pop_front();
-		dialog->Destroy();
-	}
+	gtk_widget_destroy (GTK_WIDGET(m_pMenu));
 }
 
-void gcView::SetBackgroundColor(float red, float green, float blue, float alpha)
-{
-	SetRed (red);
-	SetGreen (green);
-	SetBlue (blue);
-	SetAlpha (alpha);
-}
-
-void gcView::GetBackgroundColor(double *red, double *green, double *blue, double *alpha)
-{
-	*red = GetRed ();
-	*green = GetGreen ();
-	*blue = GetBlue ();
-	*alpha = GetAlpha ();
-}
-
-void gcView::GetRotation(double *psi, double *theta, double *phi)
-{
-	*psi = GetPsi ();
-	*theta = GetTheta ();
-	*phi = GetPhi ();
-}
-
-bool gcView::LoadOld(xmlNodePtr node)
+bool gcView::LoadOld (xmlNodePtr node)
 {
 	char *txt;
 	xmlNodePtr child = node->children;
-	while(child)
-	{
-		if (!strcmp((gchar*)child->name, "orientation"))
-		{
-			txt = (char*)xmlNodeGetContent(child);
-			if (txt)
-			{
+	while(child) {
+		if (!strcmp ((gchar*) child->name, "orientation")) {
+			txt = (char*) xmlNodeGetContent (child);
+			if (txt) {
 				double y, t, h;
-				sscanf(txt, "%lg %lg %lg", &y, &t, &h);
+				sscanf (txt, "%lg %lg %lg", &y, &t, &h);
 				SetRotation (y, t, h);
-				xmlFree(txt);
+				xmlFree (txt);
 			}
-		}
-		else if (!strcmp((gchar*)child->name, "background"))
-		{
-			txt = (char*)xmlNodeGetContent(child);
-			if (txt)
-			{
+		} else if (!strcmp ((gchar*) child->name, "background")) {
+			txt = (char*) xmlNodeGetContent (child);
+			if (txt) {
 				float b, r, g, a;
-				sscanf(txt, "%g %g %g %g", &b, &r, &g, &a);
+				sscanf (txt, "%g %g %g %g", &b, &r, &g, &a);
 				SetRed (r);
 				SetGreen (g);
 				SetBlue (b);
