@@ -28,6 +28,7 @@
 #include "view.h"
 #include "window.h"
 #include <gcugtk/filechooser.h>
+#include <gcugtk/message.h>
 #include <gcu/loader.h>
 #include <glib/gi18n.h>
 #include <clocale>
@@ -82,7 +83,7 @@ void gsvApplication::OnQuit ()
 	}
 }
 
-bool gsvApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, G_GNUC_UNUSED GtkWindow *window, Document *Doc)
+bool gsvApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *Doc)
 {
 	gsvDocument *pDoc = dynamic_cast <gsvDocument *> (Doc);
 	if(bSave) {
@@ -93,10 +94,9 @@ bool gsvApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			char *unescaped = g_uri_unescape_string (filename, NULL);
 			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), unescaped);
 			g_free (unescaped);
-			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-			gtk_window_set_icon_name (GTK_WINDOW (Box), "gspectrum");
-			result = gtk_dialog_run (Box);
-			gtk_widget_destroy (GTK_WIDGET (Box));
+			gcugtk::Message *box = new gcugtk::Message (this, message, GTK_MESSAGE_QUESTION,
+			                                            GTK_BUTTONS_YES_NO, window);
+			result = box->Run ();
 			g_free (message);
 		}
 		if (result == GTK_RESPONSE_YES) {
@@ -119,12 +119,12 @@ bool gsvApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		GtkRecentData data;
 		data.display_name = g_strdup (pDoc->GetTitle ().c_str ());
 		if (*data.display_name == 0) {
+			g_free (data.display_name);
 			char *title = g_path_get_basename (filename);
 			char *buf = g_uri_unescape_string (title, NULL);
 			g_free (title);
 			data.display_name = buf;
 			pDoc->SetTitle (data.display_name);
-			g_free (buf);
 		}
 		char *dirname = g_path_get_dirname (filename);
 		SetCurDir (dirname);
