@@ -1,14 +1,14 @@
 // -*- C++ -*-
 
-/* 
+/*
  * Gnome Chemistry Utils
  * programs/spectra/document.cc
  *
- * Copyright (C) 2007-2010 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2007-2011Jean Bréfort <jean.brefort@normalesup.org>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -27,12 +27,12 @@
 #include "application.h"
 #include "view.h"
 #include "window.h"
-#include <glib/gutils.h>
+#include <glib.h>
 
 using namespace gcu;
 using namespace std;
 
-gsvDocument::gsvDocument (gsvApplication *App): SpectrumDocument (App, new gsvView (this))
+gsvDocument::gsvDocument (gsvApplication *App): gcugtk::SpectrumDocument (App, new gsvView (this))
 {
 }
 
@@ -42,7 +42,7 @@ gsvDocument::~gsvDocument ()
 
 void gsvDocument::SetTitle (char const *title)
 {
-	gcu::SpectrumDocument::SetTitle (title);
+	gcugtk::SpectrumDocument::SetTitle (title);
 	gsvView *view = dynamic_cast <gsvView *> (m_View);
 	if (view && view->GetWindow ())
 		view->GetWindow ()->SetTitle (title);
