@@ -8,7 +8,7 @@
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -28,6 +28,7 @@
 #include "gchemtable-data.h"
 #include "gchemtable-elt.h"
 #include <gcugtk/filechooser.h>
+#include <gcugtk/message.h>
 #include <gcugtk/ui-builder.h>
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
@@ -103,24 +104,24 @@ static void on_help (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
 	app->OnHelp ();
 }
 
-static void on_web (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
+static void on_web (GtkWidget *widget, GChemTableApp *app)
 {
-	app->OnWeb ();
+	app->OnWeb (gtk_widget_get_screen (widget));
 }
 
-static void on_mail (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
+static void on_mail (GtkWidget *widget, GChemTableApp *app)
 {
-	app->OnMail ();
+	app->OnMail (gtk_widget_get_screen (widget));
 }
 
-static void on_live_assistance (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
+static void on_live_assistance (GtkWidget *widget, GChemTableApp *app)
 {
-	app->OnLiveAssistance ();
+	app->OnLiveAssistance (gtk_widget_get_screen (widget));
 }
 
-static void on_bug (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
+static void on_bug (GtkWidget *widget, GChemTableApp *app)
 {
-	app->OnBug ();
+	app->OnBug (gtk_widget_get_screen (widget));
 }
 
 static void on_about (G_GNUC_UNUSED GtkWidget *widget, GChemTableApp *app)
@@ -294,6 +295,7 @@ GChemTableApp::GChemTableApp (): gcugtk::Application ("gchemtable")
 {
 	GtkWidget *grid;
 
+	gcu::Element::LoadAllData ();
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (window), _("Periodic table of the elements"));
 	gtk_window_set_icon_name (GTK_WINDOW (window), GetIconName ().c_str ());
@@ -334,7 +336,6 @@ GChemTableApp::GChemTableApp (): gcugtk::Application ("gchemtable")
 	for (int i = 0; i < 118; i++)
 		Pages[i] = NULL;
 
-	gcu::Element::LoadAllData ();
 	gcu_periodic_set_tips (periodic, GCU_PERIODIC_TIP_STANDARD);
 	colorschemes["none"] = GCU_PERIODIC_COLOR_NONE;
 	colorschemes["default"] = GCU_PERIODIC_COLOR_DEFAULT;
@@ -395,7 +396,7 @@ void GChemTableApp::OnAbout ()
 	const gchar * license =
 		"This program is free software; you can redistribute it and/or\n"
 		"modify it under the terms of the GNU General Public License as\n"
-		"published by the Free Software Foundation; either version 2 of the\n"
+		"published by the Free Software Foundation; either version 3 of the\n"
 		"License, or (at your option) any later version.\n\n"
 		"This program is distributed in the hope that it will be useful,\n"
 		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
@@ -727,10 +728,9 @@ bool GChemTableApp::FileProcess (const gchar* filename, const gchar* mime_type,
 			char *unescaped = g_uri_unescape_string (filename, NULL);
 			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), unescaped);
 			g_free (unescaped);
-			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-			gtk_window_set_icon_name (GTK_WINDOW (Box), "gspectrum");
-			result = gtk_dialog_run (Box);
-			gtk_widget_destroy (GTK_WIDGET (Box));
+			gcugtk::Message *box = new gcugtk::Message (this, message, GTK_MESSAGE_QUESTION,
+			                                            GTK_BUTTONS_YES_NO, window);
+			result = box->Run ();
 			g_free (message);
 		}
 		if (result == GTK_RESPONSE_YES) {
